# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the iosxr_bgp_global module
"""


class Bgp_globalArgs(object):  # pylint: disable=R0903
    """The arg spec for the iosxr_bgp_global module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "as_number": {"type": "str"},
                "bfd": {
                    "type": "dict",
                    "options": {
                        "minimum_interval": {"type": "int"},
                        "multiplier": {"type": "int"},
                    },
                },
                "bgp": {
                    "type": "dict",
                    "options": {
                        "as_path_loopcheck": {"type": "bool"},
                        "auto_policy_soft_reset": {
                            "type": "dict",
                            "options": {"disable": {"type": "bool"}},
                        },
                        "bestpath": {
                            "type": "dict",
                            "options": {
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "ignore": {"type": "bool"},
                                        "multipath_relax": {"type": "bool"},
                                    },
                                },
                                "aigp": {
                                    "type": "dict",
                                    "options": {"ignore": {"type": "bool"}},
                                },
                                "med": {
                                    "type": "dict",
                                    "options": {
                                        "always": {"type": "bool"},
                                        "confed": {"type": "bool"},
                                        "missing_as_worst": {"type": "bool"},
                                    },
                                },
                                "compare_routerid": {"type": "bool"},
                                "cost_community": {
                                    "type": "dict",
                                    "options": {"ignore": {"type": "bool"}},
                                },
                                "origin_as": {
                                    "type": "dict",
                                    "options": {
                                        "use": {
                                            "type": "dict",
                                            "options": {
                                                "validity": {"type": "bool"},
                                            },
                                        },
                                        "allow": {
                                            "type": "dict",
                                            "options": {
                                                "invalid": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "cluster_id": {"type": "str"},
                        "confederation": {
                            "type": "dict",
                            "options": {
                                "identifier": {"type": "int"},
                                "peers": {"type": "list", "elements": "int"},
                            },
                        },
                        "default": {
                            "type": "dict",
                            "options": {"local_preference": {"type": "int"}},
                        },
                        "enforce_first_as": {
                            "type": "dict",
                            "options": {"disable": {"type": "bool"}},
                        },
                        "fast_external_fallover": {
                            "type": "dict",
                            "options": {"disable": {"type": "bool"}},
                        },
                        "graceful_restart": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "graceful_reset": {"type": "bool"},
                                "restart_time": {"type": "int"},
                                "purge_time": {"type": "int"},
                                "stalepath_time": {"type": "int"},
                            },
                        },
                        "install": {
                            "type": "dict",
                            "options": {"diversion": {"type": "bool"}},
                        },
                        "log": {
                            "type": "dict",
                            "options": {
                                "log_message": {
                                    "type": "dict",
                                    "options": {"disable": {"type": "bool"}},
                                },
                                "neighbor": {
                                    "type": "dict",
                                    "options": {
                                        "changes": {
                                            "type": "dict",
                                            "options": {
                                                "detail": {"type": "bool"},
                                                "disable": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "maximum": {
                            "type": "dict",
                            "options": {"neighbor": {"type": "int"}},
                        },
                        "multipath": {
                            "type": "dict",
                            "options": {
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "ignore": {
                                            "type": "dict",
                                            "options": {
                                                "onwards": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "origin_as": {
                            "type": "dict",
                            "options": {
                                "validation": {
                                    "type": "dict",
                                    "options": {
                                        "disable": {"type": "bool"},
                                        "signal": {
                                            "type": "dict",
                                            "options": {
                                                "ibgp": {"type": "bool"},
                                            },
                                        },
                                        "time": {
                                            "type": "dict",
                                            "options": {
                                                "time_off": {"type": "bool"},
                                                "time_in_second": {
                                                    "type": "int",
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "redistribute_internal": {"type": "bool"},
                        "router_id": {"type": "str"},
                        "scan_time": {"type": "int"},
                        "unsafe_ebgp_policy": {"type": "bool"},
                        "update_delay": {"type": "int"},
                    },
                },
                "default_information": {
                    "type": "dict",
                    "options": {"originate": {"type": "bool"}},
                },
                "default_metric": {"type": "int"},
                "graceful_maintenance": {
                    "type": "dict",
                    "options": {
                        "activate": {
                            "type": "str",
                            "choices": [
                                "all-neighbors",
                                "retain-routes",
                                "all-neighbors retain-routes",
                                "",
                            ],
                        },
                    },
                },
                "ibgp": {
                    "type": "dict",
                    "options": {
                        "policy": {
                            "type": "dict",
                            "options": {
                                "out": {
                                    "type": "dict",
                                    "options": {
                                        "enforce_modifications": {
                                            "type": "bool",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
                "mpls": {
                    "type": "dict",
                    "options": {
                        "activate": {
                            "type": "dict",
                            "options": {"interface": {"type": "str"}},
                        },
                    },
                },
                "mvpn": {"type": "bool"},
                "neighbors": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "neighbor_address": {
                            "type": "str",
                            "aliases": ["neighbor"],
                            "required": True,
                        },
                        "advertisement_interval": {"type": "int"},
                        "bfd": {
                            "type": "dict",
                            "options": {
                                "fast_detect": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "disable", "strict_mode"],
                                    ],
                                    "options": {
                                        "set": {"type": "bool"},
                                        "disable": {"type": "bool"},
                                        "strict_mode": {"type": "bool"},
                                    },
                                },
                                "multiplier": {"type": "int"},
                                "minimum_interval": {"type": "int"},
                            },
                        },
                        "bmp_activate": {
                            "type": "dict",
                            "options": {"server": {"type": "int"}},
                        },
                        "capability": {
                            "type": "dict",
                            "options": {
                                "additional_paths": {
                                    "type": "dict",
                                    "options": {
                                        "send": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["set", "disable"],
                                            ],
                                            "options": {
                                                "set": {"type": "bool"},
                                                "disable": {"type": "bool"},
                                            },
                                        },
                                        "receive": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["set", "disable"],
                                            ],
                                            "options": {
                                                "set": {"type": "bool"},
                                                "disable": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "suppress": {
                                    "type": "dict",
                                    "options": {
                                        "four_byte_AS": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                            },
                                        },
                                        "all": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["set", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                                "set": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "cluster_id": {"type": "str"},
                        "description": {"type": "str"},
                        "dmz_link_bandwidth": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "dscp": {"type": "str"},
                        "ebgp_multihop": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["value", "mpls"],
                            ],
                            "options": {
                                "value": {"type": "int"},
                                "mpls": {"type": "bool"},
                            },
                        },
                        "ebgp_recv_extcommunity_dmz": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "ebgp_send_extcommunity_dmz": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "cumulatie", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "cumulatie": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "egress_engineering": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "enforce_first_as": {
                            "type": "dict",
                            "options": {"disable": {"type": "bool"}},
                        },
                        "graceful_maintenance": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "activate": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "as_prepends": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["value", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "value": {"type": "int"},
                                    },
                                },
                                "local_preference": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["value", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "value": {"type": "int"},
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                    },
                                },
                            },
                        },
                        "graceful_restart": {
                            "type": "dict",
                            "options": {
                                "restart_time": {"type": "int"},
                                "stalepath_time": {"type": "int"},
                            },
                        },
                        "ignore_connected_check": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "keychain": {
                            "type": "dict",
                            "no_log": False,
                            "mutually_exclusive": [
                                ["name", "inheritance_disable"],
                            ],
                            "options": {
                                "name": {"type": "str"},
                                "inheritance_disable": {"type": "bool"},
                            },
                        },
                        "local": {
                            "type": "dict",
                            "options": {
                                "address": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["ipv4_address", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "ipv4_address": {"type": "str"},
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                    },
                                },
                            },
                        },
                        "local_as": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["value", "inheritance_disable"],
                            ],
                            "options": {
                                "value": {"type": "int"},
                                "no_prepend": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "replace_as": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "dual_as": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "inheritance_disable": {"type": "bool"},
                            },
                        },
                        "log": {
                            "type": "dict",
                            "options": {
                                "log_message": {
                                    "type": "dict",
                                    "options": {
                                        "in": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["value", "disable", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "value": {"type": "int"},
                                                "disable": {"type": "bool"},
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "out": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["value", "disable", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "value": {"type": "int"},
                                                "disable": {"type": "bool"},
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "origin_as": {
                            "type": "dict",
                            "options": {
                                "validation": {
                                    "type": "dict",
                                    "options": {"disable": {"type": "bool"}},
                                },
                            },
                        },
                        "password": {
                            "type": "dict",
                            "no_log": False,
                            "mutually_exclusive": [
                                ["encrypted", "inheritance_disable"],
                            ],
                            "options": {
                                "encrypted": {"type": "str", "no_log": True},
                                "inheritance_disable": {
                                    "type": "bool",
                                },
                            },
                        },
                        "receive_buffer_size": {"type": "int"},
                        "remote_as": {"type": "str"},
                        "send_buffer_size": {"type": "int"},
                        "session_open_mode": {
                            "type": "str",
                            "choices": ["active-only", "both", "passive-only"],
                        },
                        "shutdown": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "tcp": {
                            "type": "dict",
                            "options": {
                                "mss": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["value", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "value": {"type": "int"},
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                    },
                                },
                            },
                        },
                        "timers": {
                            "type": "dict",
                            "options": {
                                "keepalive_time": {"type": "int"},
                                "holdtime": {"type": "int"},
                            },
                        },
                        "ttl_security": {
                            "type": "dict",
                            "mutually_exclusive": [
                                ["set", "inheritance_disable"],
                            ],
                            "options": {
                                "inheritance_disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                        },
                        "update": {
                            "type": "dict",
                            "options": {
                                "in": {
                                    "type": "dict",
                                    "options": {
                                        "filtering": {
                                            "type": "dict",
                                            "options": {
                                                "attribute_filter": {
                                                    "type": "dict",
                                                    "options": {
                                                        "group": {
                                                            "type": "str",
                                                        },
                                                    },
                                                },
                                                "logging": {
                                                    "type": "dict",
                                                    "options": {
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "update_message": {
                                                    "type": "dict",
                                                    "options": {
                                                        "buffers": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "update_source": {"type": "str"},
                        "use": {
                            "type": "dict",
                            "options": {
                                "neighbor_group": {
                                    "type": "str",
                                },
                                "session_group": {
                                    "type": "str",
                                },
                            },
                        },
                    },
                },
                "nsr": {
                    "type": "dict",
                    "mutually_exclusive": [
                        ["set", "disable"],
                    ],
                    "options": {
                        "set": {"type": "bool"},
                        "disable": {"type": "bool"},
                    },
                },
                "socket": {
                    "type": "dict",
                    "options": {
                        "receive_buffer_size": {"type": "int"},
                        "send_buffer_size": {"type": "int"},
                    },
                },
                "timers": {
                    "type": "dict",
                    "options": {
                        "keepalive_time": {"type": "int"},
                        "holdtime": {"type": "int"},
                    },
                },
                "update": {
                    "type": "dict",
                    "options": {
                        "in": {
                            "type": "dict",
                            "options": {
                                "error_handling": {
                                    "type": "dict",
                                    "options": {
                                        "basic": {
                                            "type": "dict",
                                            "options": {
                                                "ebgp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "ibgp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "extended": {
                                            "type": "dict",
                                            "options": {
                                                "ebgp": {"type": "bool"},
                                                "ibgp": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "out": {
                            "type": "dict",
                            "options": {"logging": {"type": "bool"}},
                        },
                        "limit": {"type": "int"},
                    },
                },
                "rpki": {
                    "type": "dict",
                    "options": {
                        "route": {
                            "type": "dict",
                            "options": {
                                "value": {"type": "str"},
                                "max": {"type": "int"},
                                "origin": {"type": "int"},
                            },
                        },
                        "servers": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "name": {"type": "str"},
                                "purge_time": {"type": "int"},
                                "refresh_time": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "int"},
                                        "time_off": {"type": "bool"},
                                    },
                                },
                                "response_time": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "int"},
                                        "time_off": {"type": "bool"},
                                    },
                                },
                                "shutdown": {"type": "bool"},
                                "transport": {
                                    "type": "dict",
                                    "options": {
                                        "ssh": {
                                            "type": "dict",
                                            "options": {
                                                "port": {"type": "int"},
                                            },
                                        },
                                        "tcp": {
                                            "type": "dict",
                                            "options": {
                                                "port": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
                "vrfs": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "vrf": {"type": "str"},
                        "bfd": {
                            "type": "dict",
                            "options": {
                                "minimum_interval": {"type": "int"},
                                "multiplier": {"type": "int"},
                            },
                        },
                        "bgp": {
                            "type": "dict",
                            "options": {
                                "auto_policy_soft_reset": {
                                    "type": "dict",
                                    "options": {"disable": {"type": "bool"}},
                                },
                                "bestpath": {
                                    "type": "dict",
                                    "options": {
                                        "as_path": {
                                            "type": "dict",
                                            "options": {
                                                "ignore": {"type": "bool"},
                                                "multipath_relax": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "aigp": {
                                            "type": "dict",
                                            "options": {
                                                "ignore": {"type": "bool"},
                                            },
                                        },
                                        "med": {
                                            "type": "dict",
                                            "options": {
                                                "always": {"type": "bool"},
                                                "confed": {"type": "bool"},
                                                "missing_as_worst": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "compare_routerid": {"type": "bool"},
                                        "cost_community": {
                                            "type": "dict",
                                            "options": {
                                                "ignore": {"type": "bool"},
                                            },
                                        },
                                        "origin_as": {
                                            "type": "dict",
                                            "options": {
                                                "use": {
                                                    "type": "dict",
                                                    "options": {
                                                        "validity": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "allow": {
                                                    "type": "dict",
                                                    "options": {
                                                        "invalid": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "default": {
                                    "type": "dict",
                                    "options": {
                                        "local_preference": {"type": "int"},
                                    },
                                },
                                "enforce_first_as": {
                                    "type": "dict",
                                    "options": {"disable": {"type": "bool"}},
                                },
                                "fast_external_fallover": {
                                    "type": "dict",
                                    "options": {"disable": {"type": "bool"}},
                                },
                                "log": {
                                    "type": "dict",
                                    "options": {
                                        "log_message": {
                                            "type": "dict",
                                            "options": {
                                                "disable": {"type": "bool"},
                                            },
                                        },
                                        "neighbor": {
                                            "type": "dict",
                                            "options": {
                                                "changes": {
                                                    "type": "dict",
                                                    "options": {
                                                        "detail": {
                                                            "type": "bool",
                                                        },
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "multipath": {
                                    "type": "dict",
                                    "options": {
                                        "as_path": {
                                            "type": "dict",
                                            "options": {
                                                "ignore": {
                                                    "type": "dict",
                                                    "options": {
                                                        "onwards": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "redistribute_internal": {"type": "bool"},
                                "router_id": {"type": "str"},
                                "unsafe_ebgp_policy": {"type": "bool"},
                            },
                        },
                        "default_information": {
                            "type": "dict",
                            "options": {"originate": {"type": "bool"}},
                        },
                        "default_metric": {"type": "int"},
                        "mpls": {
                            "type": "dict",
                            "options": {
                                "activate": {
                                    "type": "dict",
                                    "options": {"interface": {"type": "str"}},
                                },
                            },
                        },
                        "neighbors": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "neighbor_address": {
                                    "type": "str",
                                    "aliases": ["neighbor"],
                                    "required": True,
                                },
                                "advertisement_interval": {"type": "int"},
                                "bfd": {
                                    "type": "dict",
                                    "options": {
                                        "fast_detect": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["set", "disable", "strict_mode"],
                                            ],
                                            "options": {
                                                "set": {"type": "bool"},
                                                "disable": {"type": "bool"},
                                                "strict_mode": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "multiplier": {"type": "int"},
                                        "minimum_interval": {"type": "int"},
                                    },
                                },
                                "bmp_activate": {
                                    "type": "dict",
                                    "options": {"server": {"type": "int"}},
                                },
                                "capability": {
                                    "type": "dict",
                                    "options": {
                                        "additional_paths": {
                                            "type": "dict",
                                            "options": {
                                                "send": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        ["set", "disable"],
                                                    ],
                                                    "options": {
                                                        "set": {
                                                            "type": "bool",
                                                        },
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "receive": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        ["set", "disable"],
                                                    ],
                                                    "options": {
                                                        "set": {
                                                            "type": "bool",
                                                        },
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "suppress": {
                                            "type": "dict",
                                            "options": {
                                                "four_byte_AS": {
                                                    "type": "dict",
                                                    "options": {
                                                        "set": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "all": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        ["set", "inheritance_disable"],
                                                    ],
                                                    "options": {
                                                        "inheritance_disable": {
                                                            "type": "bool",
                                                        },
                                                        "set": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "cluster_id": {"type": "str"},
                                "description": {"type": "str"},
                                "dmz_link_bandwidth": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "dscp": {"type": "str"},
                                "ebgp_multihop": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "int"},
                                        "mpls": {"type": "bool"},
                                    },
                                },
                                "ebgp_recv_extcommunity_dmz": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "ebgp_send_extcommunity_dmz": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "cumulatie": {"type": "bool"},
                                        "set": {"type": "bool"},
                                    },
                                },
                                "egress_engineering": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "enforce_first_as": {
                                    "type": "dict",
                                    "options": {"disable": {"type": "bool"}},
                                },
                                "graceful_maintenance": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "activate": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["set", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                                "set": {"type": "bool"},
                                            },
                                        },
                                        "as_prepends": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["value", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                                "value": {"type": "int"},
                                            },
                                        },
                                        "local_preference": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["value", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "value": {"type": "int"},
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                    },
                                },
                                "graceful_restart": {
                                    "type": "dict",
                                    "options": {
                                        "restart_time": {"type": "int"},
                                        "stalepath_time": {"type": "int"},
                                    },
                                },
                                "ignore_connected_check": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "keychain": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["name", "inheritance_disable"],
                                    ],
                                    "no_log": False,
                                    "options": {
                                        "name": {"type": "str"},
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                    },
                                },
                                "local": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["ipv4_address", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "ipv4_address": {
                                                    "type": "str",
                                                },
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                    },
                                },
                                "local_as": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["value", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "value": {"type": "int"},
                                        "no_prepend": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "replace_as": {
                                                    "type": "dict",
                                                    "options": {
                                                        "set": {"type": "bool"},
                                                        "dual_as": {"type": "bool"},
                                                    },
                                                },
                                            },
                                        },
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                    },
                                },
                                "log": {
                                    "type": "dict",
                                    "options": {
                                        "log_message": {
                                            "type": "dict",
                                            "options": {
                                                "in": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        ["value", "disable", "inheritance_disable"],
                                                    ],
                                                    "options": {
                                                        "value": {
                                                            "type": "int",
                                                        },
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                        "inheritance_disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                                "out": {
                                                    "type": "dict",
                                                    "mutually_exclusive": [
                                                        ["value", "disable", "inheritance_disable"],
                                                    ],
                                                    "options": {
                                                        "value": {
                                                            "type": "int",
                                                        },
                                                        "disable": {
                                                            "type": "bool",
                                                        },
                                                        "inheritance_disable": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "origin_as": {
                                    "type": "dict",
                                    "options": {
                                        "validation": {
                                            "type": "dict",
                                            "options": {
                                                "disable": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "password": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["encrypted", "inheritance_disable"],
                                    ],
                                    "no_log": False,
                                    "options": {
                                        "encrypted": {"type": "str", "no_log": True},
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                    },
                                },
                                "receive_buffer_size": {"type": "int"},
                                "remote_as": {"type": "str"},
                                "send_buffer_size": {"type": "int"},
                                "session_open_mode": {
                                    "type": "str",
                                    "choices": [
                                        "active-only",
                                        "both",
                                        "passive-only",
                                    ],
                                },
                                "shutdown": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "tcp": {
                                    "type": "dict",
                                    "options": {
                                        "mss": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                ["value", "inheritance_disable"],
                                            ],
                                            "options": {
                                                "value": {"type": "int"},
                                                "inheritance_disable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                    },
                                },
                                "timers": {
                                    "type": "dict",
                                    "options": {
                                        "keepalive_time": {"type": "int"},
                                        "holdtime": {"type": "int"},
                                    },
                                },
                                "ttl_security": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["set", "inheritance_disable"],
                                    ],
                                    "options": {
                                        "inheritance_disable": {
                                            "type": "bool",
                                        },
                                        "set": {"type": "bool"},
                                    },
                                },
                                "update": {
                                    "type": "dict",
                                    "options": {
                                        "in": {
                                            "type": "dict",
                                            "options": {
                                                "filtering": {
                                                    "type": "dict",
                                                    "options": {
                                                        "attribute_filter": {
                                                            "type": "dict",
                                                            "options": {
                                                                "group": {
                                                                    "type": "str",
                                                                },
                                                            },
                                                        },
                                                        "logging": {
                                                            "type": "dict",
                                                            "options": {
                                                                "disable": {
                                                                    "type": "bool",
                                                                },
                                                            },
                                                        },
                                                        "update_message": {
                                                            "type": "dict",
                                                            "options": {
                                                                "buffers": {
                                                                    "type": "int",
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "update_source": {"type": "str"},
                                "use": {
                                    "type": "dict",
                                    "options": {
                                        "neighbor_group": {
                                            "type": "str",
                                        },
                                        "session_group": {
                                            "type": "str",
                                        },
                                    },
                                },
                            },
                        },
                        "rd": {
                            "type": "dict",
                            "options": {"auto": {"type": "bool"}},
                        },
                        "socket": {
                            "type": "dict",
                            "options": {
                                "receive_buffer_size": {"type": "int"},
                                "send_buffer_size": {"type": "int"},
                            },
                        },
                        "timers": {
                            "type": "dict",
                            "options": {
                                "keepalive_time": {"type": "int"},
                                "holdtime": {"type": "int"},
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "type": "str",
            "choices": [
                "deleted",
                "merged",
                "replaced",
                "overridden",
                "gathered",
                "rendered",
                "parsed",
                "purged",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios hostname fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.hostname.hostname import (
    HostnameArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.hostname import (
    HostnameTemplate,
)


class HostnameFacts(object):
    """The ios hostname facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = HostnameArgs.argument_spec

    def get_hostname_data(self, connection):
        return connection.get("show running-config | section ^hostname")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Hostname network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_hostname_data(connection)

        # parse native config using the Hostname template
        hostname_parser = HostnameTemplate(lines=data.splitlines(), module=self._module)
        objs = hostname_parser.parse()

        ansible_facts["ansible_network_resources"].pop("hostname", None)

        params = utils.remove_empties(
            hostname_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )

        facts["hostname"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_l2_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_l2_interfaces
short_description: Resource module to configure L2 interfaces.
description:
  - This module provides declarative management of Layer-2 interface on Cisco
    IOS devices.
version_added: 1.0.0
author:
  - Sagar Paul (@KB-petByte)
  - Sumit Jaiswal (@justjais)
  - Nikhil Bhasin (@nickbhasin)
notes:
  - Tested against Cisco IOSv Version 15.2 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
  - The module examples uses callback plugin (stdout_callback = yaml) to generate task
    output in yaml format.
options:
  config:
    description: A dictionary of Layer-2 interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of the interface excluding any logical unit
            number, i.e GigabitEthernet0/1.
        type: str
        required: true
      access:
        description:
          - Switchport mode access command to configure the interface as a layer 2 access.
        type: dict
        suboptions:
          vlan:
            description:
              - Configure given VLAN in access port. It's used as the access VLAN ID.
            type: int
          vlan_name:
            description:
              - Set VLAN when interface is in access mode.
            type: str
      voice:
        description:
          - Switchport mode voice command to configure the interface with a voice vlan.
        type: dict
        suboptions:
          vlan:
            description:
              - Configure given voice VLAN on access port. It's used as the voice VLAN
                ID.
            type: int
          vlan_tag:
            description:
              - Set VLAN Tag.
                dot1p (Priority tagged on PVID)
                none (Don't tell telephone about voice vlan)
                untagged (Untagged on PVID)
            choices:
              - dot1p
              - none
              - untagged
            type: str
          vlan_name:
            description:
              - Set VLAN when interface is in access mode.
            type: str
      trunk:
        description:
          - Switchport mode trunk command to configure the interface as a Layer 2 trunk.
            Note The encapsulation is always set to dot1q.
        type: dict
        suboptions:
          allowed_vlans:
            description:
              - List of allowed VLANs in a given trunk port. These are the only VLANs
                that will be configured on the trunk.
            type: list
            elements: str
          native_vlan:
            description:
              - Native VLAN to be configured in trunk port. It's used as the trunk native
                VLAN ID.
            type: int
          encapsulation:
            description:
              - Trunking encapsulation when interface is in trunking mode.
            choices:
              - dot1q
              - isl
              - negotiate
            type: str
          pruning_vlans:
            description:
              - Pruning VLAN to be configured in trunk port. It's used as the trunk
                pruning VLAN ID.
            type: list
            elements: str
      mode:
        description:
          - Mode in which interface needs to be configured.
          - An interface whose trunk encapsulation is "Auto" can not be configured to
            "trunk" mode.
        type: str
        choices:
          - access
          - trunk
          - dot1q_tunnel
          - dynamic
          - dynamic_auto
          - dynamic_desirable
          - private_vlan_host
          - private_vlan_promiscuous
          - private_vlan_trunk
      xconnect:
        description:
          - Configure xconnect reference for the interface.
        type: dict
        suboptions:
          address:
            description:
              - Remote peer IP address.
            type: str
          vcid:
            description:
              - Virtual Circuit ID.
            type: int
          encapsulation:
            description:
              - Encapsulation type for xconnect.
            type: str
            choices:
              - mpls
              - l2tpv3
          manual:
            description:
              - Manually configure the xconnect.
            type: bool
          pw_class:
            description:
              - Pseudowire class name.
            type: str
          sequencing:
            description:
              - Sequencing options for the xconnect.
            type: str
            choices:
              - both
              - receive
              - transmit
      private_vlan:
        description:
          - Set the private VLAN configuration.
        type: dict
        suboptions:
          association:
            description:
              - Set the private VLAN association.
            type: bool
          host_association:
            description:
              - Set the private VLAN host association.
            type: bool
          mapping:
            description:
              - Set the private VLAN promiscuous mapping.
            type: bool
          host:
            description:
              - Set the private VLAN host association.
            type: bool
          primary_range:
            description:
              - Primary extended/normal range VLAN ID of the private VLAN promiscuous port mapping.
            type: int
          secondary_range:
            description:
              - Secondary extended/normal range VLAN ID of the private VLAN promiscuous port mapping.
            type: int
          add:
            description:
              - Add a VLAN to private VLAN list.
            type: bool
          remove:
            description:
              - Remove a VLAN from private VLAN list.
            type: bool
          secondary_vlan_id:
            description:
              - Secondary VLAN IDs of the private VLAN promiscuous port mapping.
            type: str
      app_interface:
        description:
          - Enabling port for Application Hosting (switchport app-interface)
        type: bool
      nonegotiate:
        description:
          - Device will not engage in negotiation protocol on this interface (switchport nonegotiate)
        type: bool
      vepa:
        description:
          - Reflective relay configuration (switchport vepa enabled)
        type: bool
      host:
        description:
          - Set port host (switchport host)
        type: bool
      protected:
        description:
          - Configure an interface to be a protected port (switchport protected)
        type: bool
      encapsulation:
        description:
          - Configure encapsulation for the interface.
        type: dict
        suboptions:
          type:
            description:
              - Encapsulation type (e.g., dot1q).
            type: str
          vlan_id:
            description:
              - VLAN ID for encapsulation.
            type: int
      block_options:
        description:
          - Disable forwarding of unknown uni/multi cast addresses.
        type: dict
        suboptions:
          multicast:
            description:
              - Block unknown multicast addresses
            type: bool
          unicast:
            description:
              - Block unknown unicast addresses
            type: bool
      spanning_tree:
        description:
          - Spanning tree options
        type: dict
        suboptions:
          bpdufilter:
            description: Don't send or receive BPDUs on this interface
            type: dict
            suboptions:
              enabled:
                description:
                  - Enable BPDU filtering for this interface
                type: bool
              disabled:
                description:
                  - Disable BPDU filtering for this interface
                type: bool
          bpduguard:
            description: Don't accept BPDUs on this interface
            type: dict
            suboptions:
              enabled:
                description:
                  - Enable BPDU guard for this interface
                type: bool
              disabled:
                description:
                  - Disable BPDU guard for this interface
                type: bool
          cost:
            description: Change an interface's spanning tree port path cost
            type: int
          guard:
            description: Change an interface's spanning tree guard mode
            type: dict
            suboptions:
              loop:
                description:
                  - Set guard mode to loop guard on interface
                type: bool
              none:
                description:
                  - Set guard mode to none
                type: bool
              root:
                description:
                  - Set guard mode to root guard on interface
                type: bool
          link_type:
            description: Specify a link type for spanning tree protocol use
            type: dict
            suboptions:
              point_to_point:
                description:
                  - Consider the interface as point-to-point
                type: bool
              shared:
                description:
                  - Consider the interface as shared
                type: bool
          mst:
            description: Multiple spanning tree
            type: dict
            suboptions:
              instance_range:
                description:
                  - MST instance list, example 0,2-4,6,8-12
                type: str
              cost:
                description:
                  - <1-200000000>  Change the interface spanning tree path cost for an instance
                type: str
              port_priority:
                description:
                  - Change the spanning tree port priority for an instance
                type: int
          port_priority:
            description: Change an interface's spanning tree port priority
            type: int
          portfast:
            description: Enable an interface to move directly to forwarding on link up
            type: dict
            suboptions:
              disabled:
                description:
                  - Disable portfast for this interface
                type: bool
              trunk:
                description:
                  - Enable portfast on the interface even in trunk mode
                type: bool
          rootguard:
            description: Enable root guard protection on the interface
            type: bool
          vlan:
            description: VLAN Switch Spanning Tree
            type: dict
            suboptions:
              vlan_range:
                description:
                  - MST instance list, example 1,3-5,7,9-11
                type: str
              cost:
                description:
                  - <1-200000000>  Change the interface spanning tree path cost for an instance
                type: str
              port_priority:
                description:
                  - Change the spanning tree port priority for an instance
                type: int
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | section ^interface).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and
        transform it into structured data in the format as per the resource module
        argspec and the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of
        C(running_config) option should be the same format as the output of
        command I(show running-config | include ip route|ipv6 route) executed on device.
        For state I(parsed) active connection to remote host is not required.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  media-type rj45
#  negotiation auto

- name: Merge provided configuration with device configuration
  cisco.ios.ios_l2_interfaces:
    config:
      - name: GigabitEthernet0/1
        mode: access
        access:
          vlan: 10
        voice:
          vlan: 40
      - name: GigabitEthernet0/2
        mode: trunk
        trunk:
          allowed_vlans: 10-20,40
          native_vlan: 20
          pruning_vlans: 10,20
          encapsulation: dot1q
    state: merged

# Task Output
# -----------
#
# before:
# - name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
# commands:
# - interface GigabitEthernet0/1
# - switchport access vlan 10
# - switchport voice vlan 40
# - switchport mode access
# - interface GigabitEthernet0/2
# - switchport mode trunk
# - switchport trunk encapsulation dot1q
# - switchport trunk native vlan 20
# - switchport trunk allowed vlan 10-20,40
# - switchport trunk pruning vlan 10,20
# after:
# - access:
#     vlan: 10
#   mode: access
#   name: GigabitEthernet0/1
#   voice:
#     vlan: 40
# - mode: trunk
#   name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 10-20
#     - '40'
#     encapsulation: dot1q
#     native_vlan: 20
#     pruning_vlans:
#     - '10'
#     - '20'

# After state:
# ------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 10
#  switchport voice vlan 40
#  switchport mode access
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport trunk allowed vlan 10-20,40
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 20
#  switchport trunk pruning vlan 10,20
#  switchport mode trunk
#  media-type rj45
#  negotiation auto

# Using replaced

# Before state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  media-type rj45
#  negotiation auto

- name: Replaces device configuration with provided configuration
  cisco.ios.ios_l2_interfaces:
    config:
      - name: GigabitEthernet0/2
        trunk:
          allowed_vlans: 20-25,40
          native_vlan: 20
          pruning_vlans: 10
          encapsulation: isl
    state: replaced

# Task Output
# -----------
#
# before:
# - name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
# commands:
# - interface GigabitEthernet0/2
# - no switchport access vlan
# - switchport trunk encapsulation isl
# - switchport trunk native vlan 20
# - switchport trunk allowed vlan 20-25,40
# - switchport trunk pruning vlan 10
# after:
# - access:
#     vlan: 20
#   name: GigabitEthernet0/1
# - name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 20-25
#     - '40'
#     encapsulation: isl
#     native_vlan: 20
#     pruning_vlans:
#     - '10'

# After state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport trunk allowed vlan 20-25,40
#  switchport trunk encapsulation isl
#  switchport trunk native vlan 20
#  switchport trunk pruning vlan 10
#  media-type rj45
#  negotiation auto

# Using overridden

# Before state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 20
#  media-type rj45
#  negotiation auto

- name: Override device configuration of all l2 interfaces with provided configuration
  cisco.ios.ios_l2_interfaces:
    config:
      - name: GigabitEthernet0/2
        access:
          vlan: 20
        voice:
          vlan: 40
    state: overridden

# Task Output
# -----------
#
# before:
# - name: GigabitEthernet0/1
#   trunk:
#     encapsulation: dot1q
#     native_vlan: 20
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   trunk:
#     encapsulation: dot1q
#     native_vlan: 20
# commands:
# - interface GigabitEthernet0/1
# - no switchport trunk encapsulation
# - no switchport trunk native vlan
# - interface GigabitEthernet0/2
# - switchport voice vlan 40
# - no switchport trunk encapsulation
# - no switchport trunk native vlan
# after:
# - name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   voice:
#     vlan: 40

# After state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport voice vlan 40
#  media-type rj45
#  negotiation auto

# Using deleted

# Before state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport trunk allowed vlan 20-40,60,80
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 10
#  switchport trunk pruning vlan 10
#  media-type rj45
#  negotiation auto

- name: Delete IOS L2 interfaces as in given arguments
  cisco.ios.ios_l2_interfaces:
    config:
      - name: GigabitEthernet0/1
    state: deleted

# Task Output
# -----------
#
# before:
# - access:
#     vlan: 20
#   name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 20-40
#     - '60'
#     - '80'
#     encapsulation: dot1q
#     native_vlan: 10
#     pruning_vlans:
#     - '10'
# commands:
# - interface GigabitEthernet0/1
# - no switchport access vlan
# after:
# - name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 20-40
#     - '60'
#     - '80'
#     encapsulation: dot1q
#     native_vlan: 10
#     pruning_vlans:
#     - '10'

# After state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport trunk allowed vlan 20-40,60,80
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 10
#  switchport trunk pruning vlan 10
#  media-type rj45
#  negotiation auto

# Using deleted without config - delete all configuration

# Before state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport trunk allowed vlan 20-40,60,80
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 10
#  switchport trunk pruning vlan 10
#  media-type rj45
#  negotiation auto

- name: Delete IOS L2 interfaces as in given arguments
  cisco.ios.ios_l2_interfaces:
    state: deleted

# Task Output
# -----------
#
# before:
# - access:
#     vlan: 20
#   name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 20-40
#     - '60'
#     - '80'
#     encapsulation: dot1q
#     native_vlan: 10
#     pruning_vlans:
#     - '10'
# commands:
# - interface GigabitEthernet0/1
# - no switchport access vlan
# - interface GigabitEthernet0/2
# - no switchport access vlan
# - no switchport trunk encapsulation
# - no switchport trunk native vlan
# - no switchport trunk allowed vlan
# - no switchport trunk pruning vlan
# after:
# - name: GigabitEthernet0/1
# - name: GigabitEthernet0/2

# After state:
# -------------
#
# viosl2#show running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  media-type rj45
#  negotiation auto

# Using gathered

# Before state:
# -------------
#
# vios#sh running-config | section ^interface
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport trunk allowed vlan 20-40,60,80
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 10
#  switchport trunk pruning vlan 10
#  media-type rj45
#  negotiation auto

- name: Gather facts for l2 interfaces
  cisco.ios.ios_l2_interfaces:
    config:
    state: gathered

# Task Output
# -----------
#
# gathered:
# - access:
#     vlan: 20
#   name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 20-40
#     - '60'
#     - '80'
#     encapsulation: dot1q
#     native_vlan: 10
#     pruning_vlans:
#     - '10'

# Using rendered

- name: Render the commands for provided  configuration
  cisco.ios.ios_l2_interfaces:
    config:
      - name: GigabitEthernet0/1
        access:
          vlan: 30
      - name: GigabitEthernet0/2
        trunk:
          allowed_vlans: 10-20,40
          native_vlan: 20
          pruning_vlans: 10,20
          encapsulation: dot1q
    state: rendered

# Task Output
# -----------
#
# rendered:
# - interface GigabitEthernet0/1
# - switchport access vlan 30
# - interface GigabitEthernet0/2
# - switchport trunk encapsulation dot1q
# - switchport trunk native vlan 20
# - switchport trunk allowed vlan 10-20,40
# - switchport trunk pruning vlan 10,20

# Using Parsed

# File: parsed.cfg
# ----------------
#
# interface GigabitEthernet0/1
#  description Configured by Ansible
#  switchport access vlan 20
#  negotiation auto
# interface GigabitEthernet0/2
#  description This is test
#  switchport access vlan 20
#  switchport trunk allowed vlan 20-40,60,80
#  switchport trunk encapsulation dot1q
#  switchport trunk native vlan 10
#  switchport trunk pruning vlan 10
#  media-type rj45
#  negotiation auto

- name: Parse the commands for provided configuration
  cisco.ios.ios_l2_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task Output
# -----------
#
# parsed:
# - access:
#     vlan: 20
#   name: GigabitEthernet0/1
# - access:
#     vlan: 20
#   name: GigabitEthernet0/2
#   trunk:
#     allowed_vlans:
#     - 20-40
#     - '60'
#     - '80'
#     encapsulation: dot1q
#     native_vlan: 10
#     pruning_vlans:
#     - '10'
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - interface GigabitEthernet0/2
    - switchport trunk allowed vlan 15-20,40
    - switchport trunk encapsulation dot1q
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - interface GigabitEthernet0/1
    - switchport access vlan 30
    - switchport trunk encapsulation dot1q
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.l2_interfaces.l2_interfaces import (
    L2_interfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.l2_interfaces.l2_interfaces import (
    L2_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=L2_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = L2_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2014, Brian Coca, Josh Drake, et al
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Unknown (!UNKNOWN)
name: memcached
short_description: Use memcached DB for cache
description:
  - This cache uses JSON formatted, per host records saved in memcached.
requirements:
  - memcache (python lib)
options:
  _uri:
    description:
      - List of connection information for the memcached DBs.
    default: ['127.0.0.1:11211']
    type: list
    elements: string
    env:
      - name: ANSIBLE_CACHE_PLUGIN_CONNECTION
    ini:
      - key: fact_caching_connection
        section: defaults
  _prefix:
    description: User defined prefix to use when creating the DB entries.
    type: string
    default: ansible_facts
    env:
      - name: ANSIBLE_CACHE_PLUGIN_PREFIX
    ini:
      - key: fact_caching_prefix
        section: defaults
  _timeout:
    default: 86400
    type: integer
        # TODO: determine whether it is OK to change to: type: float
    description: Expiration timeout in seconds for the cache plugin data. Set to 0 to never expire.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_TIMEOUT
    ini:
      - key: fact_caching_timeout
        section: defaults
"""

import collections
import os
import time
from collections.abc import MutableSet
from itertools import chain
from multiprocessing import Lock

from ansible.errors import AnsibleError
from ansible.plugins.cache import BaseCacheModule
from ansible.utils.display import Display

try:
    import memcache

    HAS_MEMCACHE = True
except ImportError:
    HAS_MEMCACHE = False

display = Display()


class ProxyClientPool:
    """
    Memcached connection pooling for thread/fork safety. Inspired by py-redis
    connection pool.

    Available connections are maintained in a deque and released in a FIFO manner.
    """

    def __init__(self, *args, **kwargs):
        self.max_connections = kwargs.pop("max_connections", 1024)
        self.connection_args = args
        self.connection_kwargs = kwargs
        self.reset()

    def reset(self):
        self.pid = os.getpid()
        self._num_connections = 0
        self._available_connections = collections.deque(maxlen=self.max_connections)
        self._locked_connections = set()
        self._lock = Lock()

    def _check_safe(self):
        if self.pid != os.getpid():
            with self._lock:
                if self.pid == os.getpid():
                    # bail out - another thread already acquired the lock
                    return
                self.disconnect_all()
                self.reset()

    def get_connection(self):
        self._check_safe()
        try:
            connection = self._available_connections.popleft()
        except IndexError:
            connection = self.create_connection()
        self._locked_connections.add(connection)
        return connection

    def create_connection(self):
        if self._num_connections >= self.max_connections:
            raise RuntimeError("Too many memcached connections")
        self._num_connections += 1
        return memcache.Client(*self.connection_args, **self.connection_kwargs)

    def release_connection(self, connection):
        self._check_safe()
        self._locked_connections.remove(connection)
        self._available_connections.append(connection)

    def disconnect_all(self):
        for conn in chain(self._available_connections, self._locked_connections):
            conn.disconnect_all()

    def __getattr__(self, name):
        def wrapped(*args, **kwargs):
            return self._proxy_client(name, *args, **kwargs)

        return wrapped

    def _proxy_client(self, name, *args, **kwargs):
        conn = self.get_connection()

        try:
            return getattr(conn, name)(*args, **kwargs)
        finally:
            self.release_connection(conn)


class CacheModuleKeys(MutableSet):
    """
    A set subclass that keeps track of insertion time and persists
    the set in memcached.
    """

    PREFIX = "ansible_cache_keys"

    def __init__(self, cache, *args, **kwargs):
        self._cache = cache
        self._keyset = dict(*args, **kwargs)

    def __contains__(self, key):
        return key in self._keyset

    def __iter__(self):
        return iter(self._keyset)

    def __len__(self):
        return len(self._keyset)

    def add(self, value):
        self._keyset[value] = time.time()
        self._cache.set(self.PREFIX, self._keyset)

    def discard(self, value):
        del self._keyset[value]
        self._cache.set(self.PREFIX, self._keyset)

    def remove_by_timerange(self, s_min, s_max):
        for k in list(self._keyset.keys()):
            t = self._keyset[k]
            if s_min < t < s_max:
                del self._keyset[k]
        self._cache.set(self.PREFIX, self._keyset)


class CacheModule(BaseCacheModule):
    def __init__(self, *args, **kwargs):
        connection = ["127.0.0.1:11211"]

        super().__init__(*args, **kwargs)
        if self.get_option("_uri"):
            connection = self.get_option("_uri")
        self._timeout = self.get_option("_timeout")
        self._prefix = self.get_option("_prefix")

        if not HAS_MEMCACHE:
            raise AnsibleError("python-memcached is required for the memcached fact cache")

        self._cache = {}
        self._db = ProxyClientPool(connection, debug=0)
        self._keys = CacheModuleKeys(self._db, self._db.get(CacheModuleKeys.PREFIX) or [])

    def _make_key(self, key):
        return f"{self._prefix}{key}"

    def _expire_keys(self):
        if self._timeout > 0:
            expiry_age = time.time() - self._timeout
            self._keys.remove_by_timerange(0, expiry_age)

    def get(self, key):
        if key not in self._cache:
            value = self._db.get(self._make_key(key))
            # guard against the key not being removed from the keyset;
            # this could happen in cases where the timeout value is changed
            # between invocations
            if value is None:
                self.delete(key)
                raise KeyError
            self._cache[key] = value

        return self._cache.get(key)

    def set(self, key, value):
        self._db.set(self._make_key(key), value, time=self._timeout, min_compress_len=1)
        self._cache[key] = value
        self._keys.add(key)

    def keys(self):
        self._expire_keys()
        return list(iter(self._keys))

    def contains(self, key):
        self._expire_keys()
        return key in self._keys

    def delete(self, key):
        del self._cache[key]
        self._db.delete(self._make_key(key))
        self._keys.discard(key)

    def flush(self):
        for key in self.keys():
            self.delete(key)

    def copy(self):
        return self._keys.copy()

    def __getstate__(self):
        return dict()

    def __setstate__(self, data):
        self.__init__()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import errno
import os

from ansible.utils.display import Display
from ansible_collections.azure.azcollection.plugins.plugin_utils import constants as consts
from ansible.errors import AnsibleError


display = Display()


def make_dirs_for_file(file_path):
    if not os.path.exists(file_path):
        mkdir_p(os.path.dirname(file_path))


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:  # Python <= 2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def delete_file(file_path, message, warning=False):
    # pylint: disable=broad-except
    if os.path.isfile(file_path):
        try:
            os.remove(file_path)
        except Exception as e:
            if warning:
                display(message)
            else:
                raise AnsibleError(message + "Error: " + str(e)) from e


def delete_folder(dir_path, message, warning=False):
    # pylint: disable=broad-except
    if os.path.isdir(dir_path):
        try:
            os.rmdir(dir_path)
        except Exception as e:
            if warning:
                display(message)
            else:
                raise AnsibleError(message + "Error: " + str(e)) from e


def create_directory(file_path, error_message):
    try:
        os.makedirs(file_path)
    except Exception as e:
        raise AnsibleError(error_message + "Error: " + str(e)) from e


def write_to_file(file_path, mode, content, error_message, encoding=None):
    # pylint: disable=unspecified-encoding
    try:
        if encoding:
            with open(file_path, mode, encoding=encoding) as f:
                f.write(content)
        else:
            with open(file_path, mode) as f:
                f.write(content)
    except Exception as e:
        raise AnsibleError(error_message + "Error: " + str(e)) from e


def get_line_that_contains(substring, lines):
    for line in lines:
        if substring in line:
            return line
    return None


def remove_invalid_characters_foldername(folder_name):
    new_foldername = ""
    for c in folder_name:
        if c not in consts.WINDOWS_INVALID_FOLDERNAME_CHARS:
            new_foldername += c
    return new_foldername



#!/usr/bin/python
#
# Copyright (c) 2020 Suyeb Ansari(@suyeb786), Pallavi Chaudhari(@PallaviC2510)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = \
    '''
---
module: azure_rm_backupazurevm_info
version_added: '1.1.0'
short_description: Back up an Azure Virtual Machine using Azure Backup Information
description:
    - Get Recovery point details for protected items.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    recovery_vault_name:
        description:
            - The name of the Azure Recovery Service Vault.
        required: true
        type: str
    resource_id:
        description:
            - Azure Virtual Machine Resource ID.
        required: true
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Suyeb Ansari (@suyeb786)
    - Pallavi Chaudhari (@PallaviC2510)

'''

EXAMPLES = \
    '''
- name: Get Recovery Point Details
  azure_rm_backupazurevm_info:
    resource_group: 'my-rg'
    recovery_vault_name: 'testVault'
    resource_id: "/subscriptions/<sub-id>/resourceGroups/my-rg/providers/Microsoft.Compute/virtualMachines/testVM"
'''

RETURN = \
    '''
id:
    description:
        - VM Protection details.
    returned: always
    type: str
    sample: '{"response":{"id":"protection_id","name":"protection_item_name","properties":{}}}'
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
import json


class Actions:
    (NoAction, Create, Update, Delete) = range(4)


class BackupAzureVMInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            recovery_vault_name=dict(
                type='str',
                required=True
            ),
            resource_id=dict(
                type='str',
                required=True
            )
        )

        self.resource_group = None
        self.recovery_vault_name = None
        self.resource_id = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.url = None
        self.status_code = [200, 201, 202, 204]
        self.to_do = Actions.NoAction

        self.body = {}
        self.query_parameters = {}
        self.query_parameters['api-version'] = '2019-05-13'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(BackupAzureVMInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                supports_check_mode=True,
                                                supports_tags=True)

    def get_url(self):
        return '/subscriptions' + '/' + self.subscription_id \
               + '/resourceGroups' + '/' + self.resource_group + '/providers' \
               + '/Microsoft.RecoveryServices' + '/vaults' + '/' \
               + self.recovery_vault_name \
               + '/backupFabrics/Azure/protectionContainers/' \
               + 'iaasvmcontainer;iaasvmcontainerv2;' + self.parse_resource_to_dict(self.resource_id)['resource_group'] \
               + ';' + self.parse_resource_to_dict(self.resource_id)['name'] + '/protectedItems/' \
               + 'vm;iaasvmcontainerv2;' + self.parse_resource_to_dict(self.resource_id)['resource_group'] + ';' \
               + self.parse_resource_to_dict(self.resource_id)['name']

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)
        self.url = self.get_url()

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        response = self.get_recovery_point_info()
        self.results['response'] = response
        return self.results

    def get_recovery_point_info(self):

        # self.log('Fetching protection details for the Azure Virtual Machine {0}'.format(self.))

        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error in fetching recovery point.')
            self.fail('Error in fetching recovery point {0}'.format(str(e)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response


def main():
    BackupAzureVMInfo()


if __name__ == '__main__':
    main()



# Copyright (C) 2017 Lenovo, Inc.
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard Pylxca documentation fragment
    DOCUMENTATION = r"""
author:
  - Naval Patel (@navalkp)
  - Prashant Bhosale (@prabhosa)

options:
  login_user:
    description:
      - The username for use in HTTP basic authentication.
    type: str
    required: true

  login_password:
    description:
      - The password for use in HTTP basic authentication.
    type: str
    required: true

  auth_url:
    description:
      - Lxca HTTPS full web address.
    type: str
    required: true

requirements:
  - pylxca

notes:
  - Additional detail about pylxca can be found at U(https://github.com/lenovo/pylxca).
  - Playbooks using these modules can be found at U(https://github.com/lenovo/ansible.lenovo-lxca).
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Gaspard Micol (@gmicol) <gmicol@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: ndo_virtual_port_channel_interface
short_description: Manage Virtual Port Channel Interfaces on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Virtual Port Channel Interfaces on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.2 (NDO v4.4) and later.
author:
- Gaspard Micol (@gmicol)
options:
  template:
    description:
    - The name of the Fabric Resource template.
    - This parameter or O(template_id) is required.
    type: str
    aliases: [ fabric_resource_template ]
  template_id:
    description:
    - The ID of the Fabric Policy template.
    - This parameter or O(template) is required.
    type: str
    aliases: [ fabric_resource_template_id ]
  name:
    description:
    - The name of the Virtual Port Channel Interface.
    type: str
    aliases: [ virtual_port_channel_interface, virtual_port_channel, vpc ]
  uuid:
    description:
    - The UUID of the Virtual Port Channel Interface.
    - This parameter can be used instead of O(virtual_port_channel_interface)
      when an existing Virtual Port Channel Interface is updated.
    - This parameter is required when parameter O(name) is updated.
    type: str
    aliases: [ virtual_port_channel_interface_uuid, virtual_port_channel_uuid, vpc_uuid ]
  description:
    description:
    - The description of the Virtual Port Channel Interface.
    type: str
  node_1:
    description:
    - The first node ID.
    type: str
  node_2:
    description:
    - The second node ID.
    type: str
  interfaces_node_1:
    description:
    - The list of used Interface IDs for the first node.
    - Ranges of Interface IDs can be used.
    - This parameter is required when creating a new Virtual Port Channel Interface.
    type: list
    elements: str
    aliases: [ interfaces_1, members_1 ]
  interfaces_node_2:
    description:
    - The list of used Interface IDs for the second node.
    - Ranges of Interface IDs can be used.
    - This parameter is required when creating a new Virtual Port Channel Interface.
    - If O(interfaces_node_2=[mirror]) and O(interfaces_node_1) is clearly defined,
      the interfaces of O(interfaces_node_1) will be mirrored in O(interfaces_node_2).
    type: list
    elements: str
    aliases: [ interfaces_2, members_2 ]
  interface_policy_group_uuid:
    description:
    - The UUID of the Port Channel Interface Policy Group.
    - This parameter is required when creating a new Virtual Port Channel Interface.
    type: str
    aliases: [ policy_uuid, interface_policy_uuid, interface_setting_uuid ]
  interface_policy_group:
    description:
    - The Port Channel Interface Policy Group.
    - This parameter can be used instead of O(interface_policy_group_uuid).
    type: dict
    suboptions:
      name:
        description:
        - The name of the Interface Policy Group.
        type: str
        required: true
      template:
        description:
        - The name of the template in which the Interface Policy Group has been created.
        type: str
        required: true
    aliases: [ policy, interface_policy, interface_setting ]
  interface_descriptions:
    description:
    - The list of interface descriptions of the Virtual Port Channel Interface.
    - Providing a new list of O(interface_descriptions) will completely
      replace an existing one from the Virtual Port Channel Interface.
    - Providing an empty list will remove the O(interface_descriptions=[])
      from the Virtual Port Channel Interface.
    type: list
    elements: dict
    suboptions:
      node:
        description:
        - The node ID.
        type: str
        required: true
      interface_id:
        description:
        - The interface ID or a range of interface IDs.
        - Using a range of interface IDs will
          apply the same O(interface_descriptions.description) for every ID in range.
        type: str
        required: true
      description:
        description:
        - The description of the interface or group of interfaces.
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Resource template.
- The O(interface_policy_group) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_interface_setting) to create the Interface Policy Group of type Port Channel.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_interface_setting
"""

EXAMPLES = r"""
- name: Create a new Virtual Port Channel Interface
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    description: My Ansible Port Channel
    name: ansible_virtual_port_channel_interface
    node_1: 101
    node_2: 102
    interfaces_node_1:
      - 1/1
      - 1/10-11
    interfaces_node_2:
      - 1/2
    interface_policy_group:
      name: ansible_policy_group
      template: ansible_fabric_policy_template
    interface_descriptions:
      - node: 101
        interface_id: 1/1
        description: My single Ansible Interface for first node
      - node: 101
        interface_id: 1/10-11
        description: My group of Ansible Interface for first node
      - node: 102
        interface_id: 1/2
        description: My single Ansible Interface for second node
    state: present
  register: virtual_port_channel_interface_1

- name: Update a Virtual Port Channel Interface's interfaces and their descriptions
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    description: My Ansible Port Channel
    name: ansible_virtual_port_channel_interface
    node_1: 101
    node_2: 102
    interfaces_node_1:
      - 1/1
      - 1/5-7
    interfaces_node_2:
      - 1/1-2
    interface_descriptions:
      - node: 101
        interface_id: 1/1
        description: My single unchanged Ansible Interface for first node
      - node: 101
        interface_id: 1/5-7
        description: My new group of Ansible Interface for first node
      - node: 102
        interface_id: 1/1-2
        description: My new group of Ansible Interfaces for second node
    state: present

- name: Update a Virtual Port Channel Interface by mirroring node 1 and node 2 interfaces
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    description: My Ansible Port Channel
    name: ansible_virtual_port_channel_interface
    node_1: 101
    node_2: 102
    interfaces_node_1:
      - 1/1-9
      - 1/11-15
    interfaces_node_2: mirror
    state: present

- name: Update a Virtual Port Channel Interface's name with UUID
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    name: ansible_virtual_port_channel_interface_changed
    uuid: "{{ virtual_port_channel_interface_1.current.uuid }}"
    state: present

- name: Query a Virtual Port Channel Interface with name
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    name: ansible_virtual_port_channel_interface_changed
    state: query
  register: query_name

- name: Query a Virtual Port Channel Interface with UUID
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    uuid: "{{ virtual_port_channel_interface_1.current.uuid }}"
    state: query
  register: query_uuid

- name: Query all Virtual Port Channel Interfaces in a Fabric Resource Template
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    state: query
  register: query_all

- name: Delete a Virtual Port Channel Interface with name
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    name: ansible_virtual_port_channel_interface_changed
    state: absent

- name: Delete a Virtual Port Channel Interface with UUID
  cisco.mso.ndo_virtual_port_channel_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_resource_template
    uuid: "{{ virtual_port_channel_interface_1.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
    format_interface_descriptions,
)
from ansible_collections.cisco.mso.plugins.module_utils.template import (
    MSOTemplate,
    KVPair,
)
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", aliases=["fabric_resource_template"]),
        template_id=dict(type="str", aliases=["fabric_resource_template_id"]),
        name=dict(type="str", aliases=["virtual_port_channel_interface", "virtual_port_channel", "vpc"]),
        uuid=dict(type="str", aliases=["virtual_port_channel_interface_uuid", "virtual_port_channel_uuid", "vpc_uuid"]),
        description=dict(type="str"),
        node_1=dict(type="str"),
        node_2=dict(type="str"),
        interfaces_node_1=dict(type="list", elements="str", aliases=["interfaces_1", "members_1"]),
        interfaces_node_2=dict(type="list", elements="str", aliases=["interfaces_2", "members_2"]),
        interface_policy_group=dict(
            type="dict",
            options=dict(
                name=dict(type="str", required=True),
                template=dict(type="str", required=True),
            ),
            aliases=["policy", "interface_policy", "interface_setting"],
        ),
        interface_policy_group_uuid=dict(type="str", aliases=["policy_uuid", "interface_policy_uuid", "interface_setting_uuid"]),
        interface_descriptions=dict(
            type="list",
            elements="dict",
            options=dict(
                node=dict(type="str", required=True),
                interface_id=dict(type="str", required=True),
                description=dict(type="str"),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
        required_one_of=[
            ("template", "template_id"),
        ],
        mutually_exclusive=[
            ("template", "template_id"),
            ("interface_policy_group", "interface_policy_group_uuid"),
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    node_1 = module.params.get("node_1")
    node_2 = module.params.get("node_2")
    interfaces_node_1 = module.params.get("interfaces_node_1")
    if isinstance(interfaces_node_1, list):
        interfaces_node_1 = ",".join(interfaces_node_1)
    interfaces_node_2 = module.params.get("interfaces_node_2")
    if isinstance(interfaces_node_2, list):
        if interfaces_node_2[0] == "mirror" and interfaces_node_1:
            interfaces_node_2 = interfaces_node_1
        else:
            interfaces_node_2 = ",".join(interfaces_node_2)
    interface_policy_group = module.params.get("interface_policy_group")
    interface_policy_group_uuid = module.params.get("interface_policy_group_uuid")
    interface_descriptions = module.params.get("interface_descriptions")
    state = module.params.get("state")

    match = None
    ops = []

    mso_template = MSOTemplate(mso, "fabric_resource", template, template_id)
    mso_template.validate_template("fabricResource")

    existing_virtual_port_channel_interfaces = mso_template.template.get("fabricResourceTemplate", {}).get("template", {}).get("virtualPortChannels", [])
    object_description = "Virtual Port Channel Interface"

    if state in ["query", "absent"] and not existing_virtual_port_channel_interfaces:
        mso.exit_json()
    elif state == "query" and not (name or uuid):
        mso.existing = existing_virtual_port_channel_interfaces
    elif existing_virtual_port_channel_interfaces and (name or uuid):
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            existing_virtual_port_channel_interfaces,
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            mso.existing = mso.previous = copy.deepcopy(match.details)

    if state != "query":
        virtual_port_channel_attrs_path = "/fabricResourceTemplate/template/virtualPortChannels/{0}".format(match.index if match else "-")

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        if interface_policy_group:
            fabric_policy_template = MSOTemplate(mso, "fabric_policy", interface_policy_group.get("template"))
            fabric_policy_template.validate_template("fabricPolicy")
            interface_policy_group_uuid = fabric_policy_template.get_interface_policy_group_uuid(interface_policy_group.get("name"))

        if match:
            mso_values = {
                "name": name,
                "description": description,
                ("node1Details", "node"): node_1,
                ("node1Details", "memberInterfaces"): interfaces_node_1,
                ("node2Details", "node"): node_2,
                ("node2Details", "memberInterfaces"): interfaces_node_2,
                "policy": interface_policy_group_uuid,
                "interfaceDescriptions": format_interface_descriptions(mso, interface_descriptions),
            }
            append_update_ops_data(ops, match.details, virtual_port_channel_attrs_path, mso_values)
            mso.sanitize(match.details, collate=True)

        else:
            mso_values = {
                "name": name,
                "description": description,
                "node1Details": {
                    "node": node_1,
                    "memberInterfaces": interfaces_node_1,
                },
                "node2Details": {
                    "node": node_2,
                    "memberInterfaces": interfaces_node_2,
                },
                "policy": interface_policy_group_uuid,
                "interfaceDescriptions": format_interface_descriptions(mso, interface_descriptions),
            }
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=virtual_port_channel_attrs_path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=virtual_port_channel_attrs_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        virtual_port_channel_interfaces = response.get("fabricResourceTemplate", {}).get("template", {}).get("virtualPortChannels", []) or []
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            virtual_port_channel_interfaces,
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            mso.existing = match.details
        else:
            mso.existing = {}
    elif module.check_mode and state != "query":
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2022 Western Digital Corporation
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ocapi_command
version_added: 6.3.0
short_description: Manages Out-Of-Band controllers using Open Composable API (OCAPI)
description:
  - Builds OCAPI URIs locally and sends them to remote OOB controllers to perform an action.
  - Manages OOB controller such as Indicator LED, Reboot, Power Mode, Firmware Update.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  category:
    required: true
    description:
      - Category to execute on OOB controller.
    type: str
  command:
    required: true
    description:
      - Command to execute on OOB controller.
    type: str
  baseuri:
    required: true
    description:
      - Base URI of OOB controller.
    type: str
  proxy_slot_number:
    description: For proxied inband requests, the slot number of the IOM. Only applies if O(baseuri) is a proxy server.
    type: int
  update_image_path:
    description:
      - For O(command=FWUpload), the path on the local filesystem of the firmware update image.
    type: str
  job_name:
    description:
      - For O(command=DeleteJob) command, the name of the job to delete.
    type: str
  username:
    required: true
    description:
      - Username for authenticating to OOB controller.
    type: str
  password:
    required: true
    description:
      - Password for authenticating to OOB controller.
    type: str
  timeout:
    description:
      - Timeout in seconds for URL requests to OOB controller.
    default: 10
    type: int

author: "Mike Moerk (@mikemoerk)"
"""

EXAMPLES = r"""
- name: Set the power state to low
  community.general.ocapi_command:
    category: Chassis
    command: PowerModeLow
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set the power state to normal
  community.general.ocapi_command:
    category: Chassis
    command: PowerModeNormal
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
- name: Set chassis indicator LED to on
  community.general.ocapi_command:
    category: Chassis
    command: IndicatorLedOn
    baseuri: "{{ baseuri }}"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
- name: Set chassis indicator LED to off
  community.general.ocapi_command:
    category: Chassis
    command: IndicatorLedOff
    baseuri: "{{ baseuri }}"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
- name: Reset Enclosure
  community.general.ocapi_command:
    category: Systems
    command: PowerGracefulRestart
    baseuri: "{{ baseuri }}"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
- name: Firmware Upload
  community.general.ocapi_command:
    category: Update
    command: FWUpload
    baseuri: "iom1.wdc.com"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
    update_image_path: "/path/to/firmware.tar.gz"
- name: Firmware Update
  community.general.ocapi_command:
    category: Update
    command: FWUpdate
    baseuri: "iom1.wdc.com"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
- name: Firmware Activate
  community.general.ocapi_command:
    category: Update
    command: FWActivate
    baseuri: "iom1.wdc.com"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
- name: Delete Job
  community.general.ocapi_command:
    category: Jobs
    command: DeleteJob
    job_name: FirmwareUpdate
    baseuri: "{{ baseuri }}"
    proxy_slot_number: 2
    username: "{{ username }}"
    password: "{{ password }}"
"""

RETURN = r"""
msg:
  description: Message with action result or error description.
  returned: always
  type: str
  sample: "Action was successful"

jobUri:
  description: URI to use to monitor status of the operation. Returned for async commands such as Firmware Update, Firmware
    Activate.
  returned: when supported
  type: str
  sample: "https://ioma.wdc.com/Storage/Devices/openflex-data24-usalp03020qb0003/Jobs/FirmwareUpdate/"

operationStatusId:
  description: OCAPI State ID (see OCAPI documentation for possible values).
  returned: when supported
  type: int
  sample: 2
"""

from urllib.parse import urljoin

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.general.plugins.module_utils._ocapi_utils import OcapiUtils

# More will be added as module features are expanded
CATEGORY_COMMANDS_ALL = {
    "Chassis": ["IndicatorLedOn", "IndicatorLedOff", "PowerModeLow", "PowerModeNormal"],
    "Systems": ["PowerGracefulRestart"],
    "Update": ["FWUpload", "FWUpdate", "FWActivate"],
    "Jobs": ["DeleteJob"],
}


def main():
    result = {}
    module = AnsibleModule(
        argument_spec=dict(
            category=dict(required=True),
            command=dict(required=True, type="str"),
            job_name=dict(type="str"),
            baseuri=dict(required=True, type="str"),
            proxy_slot_number=dict(type="int"),
            update_image_path=dict(type="str"),
            username=dict(required=True),
            password=dict(required=True, no_log=True),
            timeout=dict(type="int", default=10),
        ),
        supports_check_mode=True,
    )

    category = module.params["category"]
    command = module.params["command"]

    # admin credentials used for authentication
    creds = {"user": module.params["username"], "pswd": module.params["password"]}

    # timeout
    timeout = module.params["timeout"]

    base_uri = f"https://{module.params['baseuri']}"
    proxy_slot_number = module.params.get("proxy_slot_number")
    ocapi_utils = OcapiUtils(creds, base_uri, proxy_slot_number, timeout, module)

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=f"Invalid Category '{category}'. Valid Categories = {list(CATEGORY_COMMANDS_ALL.keys())}")

    # Check that the command is valid
    if command not in CATEGORY_COMMANDS_ALL[category]:
        module.fail_json(msg=f"Invalid Command '{command}'. Valid Commands = {CATEGORY_COMMANDS_ALL[category]}")

    # Organize by Categories / Commands
    if category == "Chassis":
        if command.startswith("IndicatorLed"):
            result = ocapi_utils.manage_chassis_indicator_led(command)
        elif command.startswith("PowerMode"):
            result = ocapi_utils.manage_system_power(command)
    elif category == "Systems":
        if command.startswith("Power"):
            result = ocapi_utils.manage_system_power(command)
    elif category == "Update":
        if command == "FWUpload":
            update_image_path = module.params.get("update_image_path")
            if update_image_path is None:
                module.fail_json(msg="Missing update_image_path.")
            result = ocapi_utils.upload_firmware_image(update_image_path)
        elif command == "FWUpdate":
            result = ocapi_utils.update_firmware_image()
        elif command == "FWActivate":
            result = ocapi_utils.activate_firmware_image()
    elif category == "Jobs":
        if command == "DeleteJob":
            job_name = module.params.get("job_name")
            if job_name is None:
                module.fail_json("Missing job_name")
            job_uri = urljoin(base_uri, f"Jobs/{job_name}")
            result = ocapi_utils.delete_job(job_uri)

    if result["ret"] is False:
        module.fail_json(msg=to_native(result["msg"]))
    else:
        del result["ret"]
        changed = result.get("changed", True)
        session = result.get("session", dict())
        kwargs = {
            "changed": changed,
            "session": session,
            "msg": "Action was successful."
            if not module.check_mode
            else result.get("msg", "No action performed in check mode."),
        }
        result_keys = [result_key for result_key in result if result_key not in kwargs]
        for result_key in result_keys:
            kwargs[result_key] = result[result_key]
        module.exit_json(**kwargs)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) Contributors to the Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Laszlo Szomor (@lszomor)
module: lvg_rename
short_description: Renames LVM volume groups
description:
  - This module renames volume groups using the C(vgchange) command.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
version_added: 7.1.0
options:
  vg:
    description:
      - The name or UUID of the source VG.
      - See V(vgrename(8\)) for valid values.
    type: str
    required: true
  vg_new:
    description:
      - The new name of the VG.
      - See V(lvm(8\)) for valid names.
    type: str
    required: true
seealso:
  - module: community.general.lvg
notes:
  - This module does not modify VG renaming-related configurations like C(fstab) entries or boot parameters.
"""

EXAMPLES = r"""
- name: Rename a VG by name
  community.general.lvg_rename:
    vg: vg_orig_name
    vg_new: vg_new_name

- name: Rename a VG by UUID
  community.general.lvg_rename:
    vg_uuid: SNgd0Q-rPYa-dPB8-U1g6-4WZI-qHID-N7y9Vj
    vg_new: vg_new_name
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lvm import vgrename_runner, vgs_runner

argument_spec = dict(
    vg=dict(type="str", required=True),
    vg_new=dict(type="str", required=True),
)


class LvgRename:
    def __init__(self, module):
        """
        Orchestrates the lvg_rename module logic.

        :param module: An AnsibleModule instance.
        """
        self.module = module
        self.result = {"changed": False}
        self.vg_list = []
        self._vgs = vgs_runner(module)
        self._vgrename = vgrename_runner(module)
        self._load_params()

    def run(self):
        """Performs the module logic."""

        self._load_vg_list()

        old_vg_exists = self._is_vg_exists(vg=self.vg)
        new_vg_exists = self._is_vg_exists(vg=self.vg_new)

        if old_vg_exists:
            if new_vg_exists:
                self.module.fail_json(msg=f"The new VG name ({self.vg_new}) is already in use.")
            else:
                self._rename_vg()
        else:
            if new_vg_exists:
                self.result["msg"] = f"The new VG ({self.vg_new}) already exists, nothing to do."
                self.module.exit_json(**self.result)
            else:
                self.module.fail_json(msg=f"Both current ({self.vg}) and new ({self.vg_new}) VG are missing.")

        self.module.exit_json(**self.result)

    def _load_params(self):
        """Load the parameters from the module."""

        self.vg = self.module.params["vg"]
        self.vg_new = self.module.params["vg_new"]

    def _load_vg_list(self):
        """Load the VGs from the system."""

        with self._vgs("noheadings separator fields") as ctx:
            dummy, vg_raw_list, dummy = ctx.run(separator=";", fields="vg_name,vg_uuid")

        for vg_info in vg_raw_list.splitlines():
            vg_name, vg_uuid = vg_info.strip().split(";")
            self.vg_list.append(vg_name)
            self.vg_list.append(vg_uuid)

    def _is_vg_exists(self, vg):
        """
        Checks VG existence by name or UUID. It removes the '/dev/' prefix before checking.

        :param vg: A string with the name or UUID of the VG.
        :returns: A boolean indicates whether the VG exists or not.
        """

        vg_found = False
        dev_prefix = "/dev/"

        if vg.startswith(dev_prefix):
            vg_id = vg[len(dev_prefix) :]
        else:
            vg_id = vg

        vg_found = vg_id in self.vg_list

        return vg_found

    def _rename_vg(self):
        """Renames the volume group."""

        if self.module._diff:
            self.result["diff"] = {"before": {"vg": self.vg}, "after": {"vg": self.vg_new}}

        if self.module.check_mode:
            self.result["msg"] = f"Running in check mode. The module would rename VG {self.vg} to {self.vg_new}."
            self.result["changed"] = True
        else:
            with self._vgrename("vg vg_new") as ctx:
                dummy, vg_rename_out, dummy = ctx.run()

            self.result["msg"] = vg_rename_out
            self.result["changed"] = True


def setup_module_object():
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    return module


def main():
    module = setup_module_object()
    lvg_rename = LvgRename(module=module)
    lvg_rename.run()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2014, Chris Schmidt <chris.schmidt () contrastsecurity.com>
#
# Built using https://github.com/hamnis/useful-scripts/blob/master/python/download-maven-artifact
# as a reference and starting point.
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: maven_artifact
short_description: Downloads an Artifact from a Maven Repository
description:
  - Downloads an artifact from a maven repository given the maven coordinates provided to the module.
  - Can retrieve snapshots or release versions of the artifact and resolve the latest available version if one is not available.
author: "Chris Schmidt (@chrisisbeef)"
requirements:
  - lxml
  - boto if using a S3 repository (V(s3://...))
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  group_id:
    type: str
    description:
      - The Maven groupId coordinate.
    required: true
  artifact_id:
    type: str
    description:
      - The maven artifactId coordinate.
    required: true
  version:
    type: str
    description:
      - The maven version coordinate.
      - Mutually exclusive with O(version_by_spec).
  version_by_spec:
    type: str
    description:
      - The maven dependency version ranges.
      - See supported version ranges on U(https://cwiki.apache.org/confluence/display/MAVENOLD/Dependency+Mediation+and+Conflict+Resolution).
      - The range type V((,1.0],[1.2,\)) and V((,1.1\),(1.1,\)) is not supported.
      - Mutually exclusive with O(version).
    version_added: '0.2.0'
  classifier:
    type: str
    description:
      - The maven classifier coordinate.
    default: ''
  extension:
    type: str
    description:
      - The maven type/extension coordinate.
    default: jar
  repository_url:
    type: str
    description:
      - The URL of the Maven Repository to download from.
      - Use V(s3://...) if the repository is hosted on Amazon S3.
      - Use V(file://...) if the repository is local.
    default: https://repo1.maven.org/maven2
  username:
    type: str
    description:
      - The username to authenticate as to the Maven Repository. Use AWS secret key of the repository is hosted on S3.
    aliases: ["aws_secret_key"]
  password:
    type: str
    description:
      - The password to authenticate with to the Maven Repository. Use AWS secret access key of the repository is hosted on
        S3.
    aliases: ["aws_secret_access_key"]
  headers:
    description:
      - Add custom HTTP headers to a request in hash/dict format.
    type: dict
  force_basic_auth:
    description:
      - C(httplib2), the library used by the URI module only sends authentication information when a webservice responds to
        an initial request with a 401 status. Since some basic auth services do not properly send a 401, logins fail. This
        option forces the sending of the Basic authentication header upon initial request.
    default: false
    type: bool
    version_added: '0.2.0'
  dest:
    type: path
    description:
      - The path where the artifact should be written to.
      - If file mode or ownerships are specified and destination path already exists, they affect the downloaded file.
    required: true
  state:
    type: str
    description:
      - The desired state of the artifact.
    default: present
    choices: [present, absent]
  timeout:
    type: int
    description:
      - Specifies a timeout in seconds for the connection attempt.
    default: 10
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated. This should only be set to V(false) when no other option exists.
    type: bool
    default: true
  client_cert:
    description:
      - PEM formatted certificate chain file to be used for SSL client authentication.
      - This file can also include the key as well, and if the key is included, O(client_key) is not required.
    type: path
    version_added: '1.3.0'
  client_key:
    description:
      - PEM formatted file that contains your private key to be used for SSL client authentication.
      - If O(client_cert) contains both the certificate and key, this option is not required.
    type: path
    version_added: '1.3.0'
  keep_name:
    description:
      - If V(true), the downloaded artifact's name is preserved, in other words the version number remains part of it.
      - This option only has effect when O(dest) is a directory and O(version) is set to V(latest) or O(version_by_spec) is
        defined.
    type: bool
    default: false
  verify_checksum:
    type: str
    description:
      - If V(never), the MD5/SHA1 checksum is never downloaded and verified.
      - If V(download), the MD5/SHA1 checksum is downloaded and verified only after artifact download. This is the default.
      - If V(change), the MD5/SHA1 checksum is downloaded and verified if the destination already exist, to verify if they
        are identical. This was the behaviour before 2.6. Since it downloads the checksum before (maybe) downloading the artifact,
        and since some repository software, when acting as a proxy/cache, return a 404 error if the artifact has not been
        cached yet, it may fail unexpectedly. If you still need it, you should consider using V(always) instead - if you deal
        with a checksum, it is better to use it to verify integrity after download.
      - V(always) combines V(download) and V(change).
    default: 'download'
    choices: ['never', 'download', 'change', 'always']
  checksum_alg:
    type: str
    description:
      - If V(md5), checksums use the MD5 algorithm. This is the default.
      - If V(sha1), checksums use the SHA1 algorithm. This can be used on systems configured to use FIPS-compliant algorithms,
        since MD5 is blocked on such systems.
    default: 'md5'
    choices: ['md5', 'sha1']
    version_added: 3.2.0
  unredirected_headers:
    type: list
    elements: str
    version_added: 5.2.0
    description:
      - A list of headers that should not be included in the redirection. This headers are sent to the C(fetch_url) function.
      - On ansible-core version 2.12 or later, the default of this option is V([Authorization, Cookie]).
      - Useful if the redirection URL does not need to have sensitive headers in the request.
      - Requires ansible-core version 2.12 or later.
  directory_mode:
    type: str
    description:
      - Filesystem permission mode applied recursively to O(dest) when it is a directory.
extends_documentation_fragment:
  - ansible.builtin.files
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Download the latest version of the JUnit framework artifact from Maven Central
  community.general.maven_artifact:
    group_id: junit
    artifact_id: junit
    dest: /tmp/junit-latest.jar

- name: Download JUnit 4.11 from Maven Central
  community.general.maven_artifact:
    group_id: junit
    artifact_id: junit
    version: 4.11
    dest: /tmp/junit-4.11.jar

- name: Download an artifact from a private repository requiring authentication
  community.general.maven_artifact:
    group_id: com.company
    artifact_id: library-name
    repository_url: 'https://repo.company.com/maven'
    username: user
    password: pass
    dest: /tmp/library-name-latest.jar

- name: Download an artifact from a private repository requiring certificate authentication
  community.general.maven_artifact:
    group_id: com.company
    artifact_id: library-name
    repository_url: 'https://repo.company.com/maven'
    client_cert: /path/to/cert.pem
    client_key: /path/to/key.pem
    dest: /tmp/library-name-latest.jar

- name: Download a WAR File to the Tomcat webapps directory to be deployed
  community.general.maven_artifact:
    group_id: com.company
    artifact_id: web-app
    extension: war
    repository_url: 'https://repo.company.com/maven'
    dest: /var/lib/tomcat7/webapps/web-app.war

- name: Keep a downloaded artifact's name, i.e. retain the version
  community.general.maven_artifact:
    version: latest
    artifact_id: spring-core
    group_id: org.springframework
    dest: /tmp/
    keep_name: true

- name: Download the latest version of the JUnit framework artifact from Maven local
  community.general.maven_artifact:
    group_id: junit
    artifact_id: junit
    dest: /tmp/junit-latest.jar
    repository_url: "file://{{ lookup('env','HOME') }}/.m2/repository"

- name: Download the latest version between 3.8 and 4.0 (exclusive) of the JUnit framework artifact from Maven Central
  community.general.maven_artifact:
    group_id: junit
    artifact_id: junit
    version_by_spec: "[3.8,4.0)"
    dest: /tmp/
"""

import hashlib
import os
import posixpath
import re
import shutil
import tempfile
import traceback
from re import match
from urllib.parse import urlparse

from ansible.module_utils.ansible_release import __version__ as ansible_version

LXML_ETREE_IMP_ERR = None
try:
    from lxml import etree

    HAS_LXML_ETREE = True
except ImportError:
    LXML_ETREE_IMP_ERR = traceback.format_exc()
    HAS_LXML_ETREE = False

BOTO_IMP_ERR = None
try:
    import boto3

    HAS_BOTO = True
except ImportError:
    BOTO_IMP_ERR = traceback.format_exc()
    HAS_BOTO = False

SEMANTIC_VERSION_IMP_ERR = None
try:
    from semantic_version import Spec, Version

    HAS_SEMANTIC_VERSION = True
except ImportError:
    SEMANTIC_VERSION_IMP_ERR = traceback.format_exc()
    HAS_SEMANTIC_VERSION = False


from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.module_utils.urls import fetch_url


def split_pre_existing_dir(dirname):
    """
    Return the first pre-existing directory and a list of the new directories that will be created.
    """
    head, tail = os.path.split(dirname)
    b_head = to_bytes(head, errors="surrogate_or_strict")
    if not os.path.exists(b_head):
        if head == dirname:
            return None, [head]
        else:
            (pre_existing_dir, new_directory_list) = split_pre_existing_dir(head)
    else:
        return head, [tail]
    new_directory_list.append(tail)
    return pre_existing_dir, new_directory_list


def adjust_recursive_directory_permissions(pre_existing_dir, new_directory_list, module, directory_args, changed):
    """
    Walk the new directories list and make sure that permissions are as we would expect
    """
    if new_directory_list:
        first_sub_dir = new_directory_list.pop(0)
        if not pre_existing_dir:
            working_dir = first_sub_dir
        else:
            working_dir = os.path.join(pre_existing_dir, first_sub_dir)
        directory_args["path"] = working_dir
        changed = module.set_fs_attributes_if_different(directory_args, changed)
        changed = adjust_recursive_directory_permissions(
            working_dir, new_directory_list, module, directory_args, changed
        )
    return changed


class Artifact:
    def __init__(self, group_id, artifact_id, version, version_by_spec, classifier="", extension="jar"):
        if not group_id:
            raise ValueError("group_id must be set")
        if not artifact_id:
            raise ValueError("artifact_id must be set")

        self.group_id = group_id
        self.artifact_id = artifact_id
        self.version = version
        self.version_by_spec = version_by_spec
        self.classifier = classifier

        if not extension:
            self.extension = "jar"
        else:
            self.extension = extension

    def is_snapshot(self):
        return self.version and self.version.endswith("SNAPSHOT")

    def path(self, with_version=True):
        base = posixpath.join(self.group_id.replace(".", "/"), self.artifact_id)
        if with_version and self.version:
            timestamp_version_match = re.match("^(.*-)?([0-9]{8}\\.[0-9]{6}-[0-9]+)$", self.version)
            if timestamp_version_match:
                base = posixpath.join(base, f"{timestamp_version_match.group(1)}SNAPSHOT")
            else:
                base = posixpath.join(base, self.version)
        return base

    def _generate_filename(self):
        filename = f"{self.artifact_id}-{self.classifier}.{self.extension}"
        if not self.classifier:
            filename = f"{self.artifact_id}.{self.extension}"
        return filename

    def get_filename(self, filename=None):
        if not filename:
            filename = self._generate_filename()
        elif os.path.isdir(filename):
            filename = os.path.join(filename, self._generate_filename())
        return filename

    def __str__(self):
        result = f"{self.group_id}:{self.artifact_id}:{self.version}"
        if self.classifier:
            result = f"{self.group_id}:{self.artifact_id}:{self.extension}:{self.classifier}:{self.version}"
        elif self.extension != "jar":
            result = f"{self.group_id}:{self.artifact_id}:{self.extension}:{self.version}"
        return result

    @staticmethod
    def parse(input):
        parts = input.split(":")
        if len(parts) >= 3:
            g = parts[0]
            a = parts[1]
            v = parts[-1]
            t = None
            c = None
            if len(parts) == 4:
                t = parts[2]
            if len(parts) == 5:
                t = parts[2]
                c = parts[3]
            return Artifact(g, a, v, c, t)
        else:
            return None


class MavenDownloader:
    def __init__(self, module, base, local=False, headers=None):
        self.module = module
        if base.endswith("/"):
            base = base.rstrip("/")
        self.base = base
        self.local = local
        self.headers = headers
        self.user_agent = f"Ansible {ansible_version} maven_artifact"
        self.latest_version_found = None
        self.metadata_file_name = "maven-metadata-local.xml" if local else "maven-metadata.xml"

    def find_version_by_spec(self, artifact):
        path = f"/{artifact.path(False)}/{self.metadata_file_name}"
        content = self._getContent(self.base + path, f"Failed to retrieve the maven metadata file: {path}")
        xml = etree.fromstring(content)
        original_versions = xml.xpath("/metadata/versioning/versions/version/text()")
        versions = []
        for version in original_versions:
            try:
                versions.append(Version.coerce(version))
            except ValueError:
                # This means that version string is not a valid semantic versioning
                pass

        parse_versions_syntax = {
            # example -> (,1.0]
            r"^\(,(?P<upper_bound>[0-9.]*)]$": "<={upper_bound}",
            # example -> 1.0
            r"^(?P<version>[0-9.]*)$": "~={version}",
            # example -> [1.0]
            r"^\[(?P<version>[0-9.]*)\]$": "=={version}",
            # example -> [1.2, 1.3]
            r"^\[(?P<lower_bound>[0-9.]*),\s*(?P<upper_bound>[0-9.]*)\]$": ">={lower_bound},<={upper_bound}",
            # example -> [1.2, 1.3)
            r"^\[(?P<lower_bound>[0-9.]*),\s*(?P<upper_bound>[0-9.]+)\)$": ">={lower_bound},<{upper_bound}",
            # example -> [1.5,)
            r"^\[(?P<lower_bound>[0-9.]*),\)$": ">={lower_bound}",
        }

        for regex, spec_format in parse_versions_syntax.items():
            regex_result = match(regex, artifact.version_by_spec)
            if regex_result:
                spec = Spec(spec_format.format(**regex_result.groupdict()))
                selected_version = spec.select(versions)

                if not selected_version:
                    raise ValueError(f"No version found with this spec version: {artifact.version_by_spec}")

                # To deal when repos on maven don't have patch number on first build (e.g. 3.8 instead of 3.8.0)
                if str(selected_version) not in original_versions:
                    selected_version.patch = None

                return str(selected_version)

        raise ValueError(f"The spec version {artifact.version_by_spec} is not supported! ")

    def find_latest_version_available(self, artifact):
        if self.latest_version_found:
            return self.latest_version_found
        path = f"/{artifact.path(False)}/{self.metadata_file_name}"
        content = self._getContent(self.base + path, f"Failed to retrieve the maven metadata file: {path}")
        xml = etree.fromstring(content)
        v = xml.xpath("/metadata/versioning/versions/version[last()]/text()")
        if v:
            self.latest_version_found = v[0]
            return v[0]

    def find_uri_for_artifact(self, artifact):
        if artifact.version_by_spec:
            artifact.version = self.find_version_by_spec(artifact)

        if artifact.version == "latest":
            artifact.version = self.find_latest_version_available(artifact)

        if artifact.is_snapshot():
            if self.local:
                return self._uri_for_artifact(artifact, artifact.version)
            path = f"/{artifact.path()}/{self.metadata_file_name}"
            content = self._getContent(self.base + path, f"Failed to retrieve the maven metadata file: {path}")
            xml = etree.fromstring(content)

            candidates = []
            for snapshot_artifact in xml.xpath("/metadata/versioning/snapshotVersions/snapshotVersion"):
                classifier = snapshot_artifact.xpath("classifier/text()")
                extension = snapshot_artifact.xpath("extension/text()")
                if (classifier[0] if classifier else "") == artifact.classifier and (
                    extension[0] if extension else ""
                ) == artifact.extension:
                    value = snapshot_artifact.xpath("value/text()")
                    updated = snapshot_artifact.xpath("updated/text()")
                    if value:
                        candidates.append((updated[0] if updated else "", value[0]))
            if candidates:
                # updated is yyyymmddHHMMSS, so lexical max == newest
                return self._uri_for_artifact(artifact, max(candidates, key=lambda item: item[0])[1])
            timestamp_xmlpath = xml.xpath("/metadata/versioning/snapshot/timestamp/text()")
            build_number_xmlpath = xml.xpath("/metadata/versioning/snapshot/buildNumber/text()")
            if timestamp_xmlpath and build_number_xmlpath:
                timestamp = timestamp_xmlpath[0]
                build_number = build_number_xmlpath[0]
                return self._uri_for_artifact(
                    artifact, artifact.version.replace("SNAPSHOT", f"{timestamp}-{build_number}")
                )

        return self._uri_for_artifact(artifact, artifact.version)

    def _uri_for_artifact(self, artifact, version=None):
        if artifact.is_snapshot() and not version:
            raise ValueError(f"Expected uniqueversion for snapshot artifact {artifact}")
        elif not artifact.is_snapshot():
            version = artifact.version
        if artifact.classifier:
            return posixpath.join(
                self.base,
                artifact.path(),
                f"{artifact.artifact_id}-{version}-{artifact.classifier}.{artifact.extension}",
            )

        return posixpath.join(self.base, artifact.path(), f"{artifact.artifact_id}-{version}.{artifact.extension}")

    # for small files, directly get the full content
    def _getContent(self, url, failmsg, force=True):
        if self.local:
            parsed_url = urlparse(url)
            if os.path.isfile(parsed_url.path):
                with open(parsed_url.path, "rb") as f:
                    return f.read()
            if force:
                raise ValueError(f"{failmsg} because can not find file: {url}")
            return None
        response = self._request(url, failmsg, force)
        if response:
            return response.read()
        return None

    # only for HTTP request
    def _request(self, url, failmsg, force=True):
        url_to_use = url
        parsed_url = urlparse(url)

        if parsed_url.scheme == "s3":
            parsed_url = urlparse(url)
            bucket_name = parsed_url.netloc
            key_name = parsed_url.path[1:]
            client = boto3.client(
                "s3",
                aws_access_key_id=self.module.params.get("username", ""),
                aws_secret_access_key=self.module.params.get("password", ""),
            )
            url_to_use = client.generate_presigned_url(
                "get_object", Params={"Bucket": bucket_name, "Key": key_name}, ExpiresIn=10
            )

        req_timeout = self.module.params.get("timeout")

        # Hack to add parameters in the way that fetch_url expects
        self.module.params["url_username"] = self.module.params.get("username", "")
        self.module.params["url_password"] = self.module.params.get("password", "")
        self.module.params["http_agent"] = self.user_agent

        kwargs = {}
        if self.module.params["unredirected_headers"]:
            kwargs["unredirected_headers"] = self.module.params["unredirected_headers"]

        response, info = fetch_url(self.module, url_to_use, timeout=req_timeout, headers=self.headers, **kwargs)

        if info["status"] == 200:
            return response
        if force:
            raise ValueError(f"{failmsg} because of {info['msg']}for URL {url_to_use}")
        return None

    def download(self, tmpdir, artifact, verify_download, filename=None, checksum_alg="md5"):
        if (not artifact.version and not artifact.version_by_spec) or artifact.version == "latest":
            artifact = Artifact(
                artifact.group_id,
                artifact.artifact_id,
                self.find_latest_version_available(artifact),
                None,
                artifact.classifier,
                artifact.extension,
            )
        url = self.find_uri_for_artifact(artifact)
        tempfd, tempname = tempfile.mkstemp(dir=tmpdir)

        try:
            # copy to temp file
            if self.local:
                parsed_url = urlparse(url)
                if os.path.isfile(parsed_url.path):
                    shutil.copy2(parsed_url.path, tempname)
                else:
                    return f"Can not find local file: {parsed_url.path}"
            else:
                response = self._request(url, f"Failed to download artifact {artifact}")
                with os.fdopen(tempfd, "wb") as f:
                    shutil.copyfileobj(response, f)

            if verify_download:
                invalid_checksum = self.is_invalid_checksum(tempname, url, checksum_alg)
                if invalid_checksum:
                    # if verify_change was set, the previous file would be deleted
                    os.remove(tempname)
                    return invalid_checksum
        except Exception as e:
            os.remove(tempname)
            raise e

        # all good, now copy temp file to target
        shutil.move(tempname, artifact.get_filename(filename))
        return None

    def is_invalid_checksum(self, file, remote_url, checksum_alg="md5"):
        if os.path.exists(file):
            local_checksum = self._local_checksum(checksum_alg, file)
            if self.local:
                parsed_url = urlparse(remote_url)
                remote_checksum = self._local_checksum(checksum_alg, parsed_url.path)
            else:
                try:
                    remote_checksum = to_text(
                        self._getContent(f"{remote_url}.{checksum_alg}", "Failed to retrieve checksum", False),
                        errors="strict",
                    )
                except UnicodeError as e:
                    return f"Cannot retrieve a valid {checksum_alg} checksum from {remote_url}: {e}"
                if not remote_checksum:
                    return f"Cannot find {checksum_alg} checksum from {remote_url}"
            try:
                # Check if remote checksum only contains md5/sha1 or md5/sha1 + filename
                _remote_checksum = remote_checksum.split(None, 1)[0]
                remote_checksum = _remote_checksum
                # remote_checksum is empty so we continue and keep original checksum string
                # This should not happen since we check for remote_checksum before
            except IndexError:
                pass
            if local_checksum.lower() == remote_checksum.lower():
                return None
            else:
                return (
                    f"Checksum does not match: we computed {local_checksum} but the repository states {remote_checksum}"
                )

        return f"Path does not exist: {file}"

    def _local_checksum(self, checksum_alg, file):
        if checksum_alg.lower() == "md5":
            hash = hashlib.md5()
        elif checksum_alg.lower() == "sha1":
            hash = hashlib.sha1()
        else:
            raise ValueError(f"Unknown checksum_alg {checksum_alg}")
        with open(file, "rb") as f:
            for chunk in iter(lambda: f.read(8192), b""):
                hash.update(chunk)
        return hash.hexdigest()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            group_id=dict(required=True),
            artifact_id=dict(required=True),
            version=dict(),
            version_by_spec=dict(),
            classifier=dict(default=""),
            extension=dict(default="jar"),
            repository_url=dict(default="https://repo1.maven.org/maven2"),
            username=dict(aliases=["aws_secret_key"]),
            password=dict(no_log=True, aliases=["aws_secret_access_key"]),
            headers=dict(type="dict"),
            force_basic_auth=dict(default=False, type="bool"),
            state=dict(default="present", choices=["present", "absent"]),
            timeout=dict(default=10, type="int"),
            dest=dict(type="path", required=True),
            validate_certs=dict(default=True, type="bool"),
            client_cert=dict(type="path"),
            client_key=dict(type="path"),
            keep_name=dict(default=False, type="bool"),
            verify_checksum=dict(default="download", choices=["never", "download", "change", "always"]),
            checksum_alg=dict(default="md5", choices=["md5", "sha1"]),
            unredirected_headers=dict(type="list", elements="str"),
            directory_mode=dict(type="str"),
        ),
        add_file_common_args=True,
        mutually_exclusive=([("version", "version_by_spec")]),
    )

    if module.params["unredirected_headers"] is None:
        # if the user did not supply unredirected params, we use the default
        module.params["unredirected_headers"] = ["Authorization", "Cookie"]

    if not HAS_LXML_ETREE:
        module.fail_json(msg=missing_required_lib("lxml"), exception=LXML_ETREE_IMP_ERR)

    if module.params["version_by_spec"] and not HAS_SEMANTIC_VERSION:
        module.fail_json(msg=missing_required_lib("semantic_version"), exception=SEMANTIC_VERSION_IMP_ERR)

    repository_url = module.params["repository_url"]
    if not repository_url:
        repository_url = "https://repo1.maven.org/maven2"
    try:
        parsed_url = urlparse(repository_url)
    except AttributeError as e:
        module.fail_json(msg=f"url parsing went wrong {e}")

    local = parsed_url.scheme == "file"

    if parsed_url.scheme == "s3" and not HAS_BOTO:
        module.fail_json(
            msg=missing_required_lib("boto3", reason="when using s3:// repository URLs"), exception=BOTO_IMP_ERR
        )

    group_id = module.params["group_id"]
    artifact_id = module.params["artifact_id"]
    version = module.params["version"]
    version_by_spec = module.params["version_by_spec"]
    classifier = module.params["classifier"]
    extension = module.params["extension"]
    headers = module.params["headers"]
    state = module.params["state"]
    dest = module.params["dest"]
    b_dest = to_bytes(dest, errors="surrogate_or_strict")
    keep_name = module.params["keep_name"]
    verify_checksum = module.params["verify_checksum"]
    verify_download = verify_checksum in ["download", "always"]
    verify_change = verify_checksum in ["change", "always"]
    checksum_alg = module.params["checksum_alg"]

    downloader = MavenDownloader(module, repository_url, local, headers)

    if not version_by_spec and not version:
        version = "latest"

    try:
        artifact = Artifact(group_id, artifact_id, version, version_by_spec, classifier, extension)
    except ValueError as e:
        module.fail_json(msg=e.args[0])

    changed = False
    prev_state = "absent"

    if dest.endswith(os.sep):
        b_dest = to_bytes(dest, errors="surrogate_or_strict")
        if not os.path.exists(b_dest):
            (pre_existing_dir, new_directory_list) = split_pre_existing_dir(dest)
            os.makedirs(b_dest)
            directory_args = module.load_file_common_arguments(module.params)
            directory_mode = module.params["directory_mode"]
            if directory_mode is not None:
                directory_args["mode"] = directory_mode
            else:
                directory_args["mode"] = None
            changed = adjust_recursive_directory_permissions(
                pre_existing_dir, new_directory_list, module, directory_args, changed
            )

    if os.path.isdir(b_dest):
        version_part = version
        if version == "latest":
            version_part = downloader.find_latest_version_available(artifact)
        elif version_by_spec:
            version_part = downloader.find_version_by_spec(artifact)

        filename = f"{artifact_id}{(f'-{version_part}' if keep_name else '')}{(f'-{classifier}' if classifier else '')}.{extension}"
        dest = posixpath.join(dest, filename)

        b_dest = to_bytes(dest, errors="surrogate_or_strict")

    if os.path.lexists(b_dest) and (
        (not verify_change)
        or not downloader.is_invalid_checksum(dest, downloader.find_uri_for_artifact(artifact), checksum_alg)
    ):
        prev_state = "present"

    if prev_state == "absent":
        try:
            download_error = downloader.download(module.tmpdir, artifact, verify_download, b_dest, checksum_alg)
            if download_error is None:
                changed = True
            else:
                module.fail_json(msg=f"Cannot retrieve the artifact to destination: {download_error}")
        except ValueError as e:
            module.fail_json(msg=e.args[0])

    file_args = module.load_file_common_arguments(module.params, path=dest)
    changed = module.set_fs_attributes_if_different(file_args, changed)
    if changed:
        module.exit_json(
            state=state,
            dest=dest,
            group_id=group_id,
            artifact_id=artifact_id,
            version=version,
            classifier=classifier,
            extension=extension,
            repository_url=repository_url,
            changed=changed,
        )
    else:
        module.exit_json(state=state, dest=dest, changed=changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_certificate_management_policy
short_description: Certificate Management Policy configuration for Cisco Intersight
description:
  - Manages Certificate Management Policy configuration on Cisco Intersight.
  - A policy to configure certificate management for Cisco Intersight managed servers.
  - This policy allows configuration of Root CA certificates and IMC certificates with private keys.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/certificatemanagement/Policies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Certificate Management Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Certificate Management Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  certificates:
    description:
      - List of certificates to manage in the policy.
      - Can include Root CA certificates and IMC certificates.
      - Only one IMC certificate can be configured per policy.
    type: list
    elements: dict
    suboptions:
      certificate_type:
        description:
          - Type of certificate to configure.
          - C(rootca) for Root CA certificates.
          - C(imc) for IMC certificates (only one allowed per policy).
        type: str
        choices: [rootca, imc]
        required: true
      certificate_name:
        description:
          - A name that helps identify a certificate.
          - Can be any string that adheres to the following constraints.
          - It should start and end with an alphanumeric character.
          - It can have underscores and hyphens.
          - It cannot be more than 30 characters.
          - Required for Root CA certificates.
        type: str
      certificate:
        description:
          - The PEM-encoded certificate data.
          - Must be base64 encoded.
          - Required for all certificate types.
        type: str
        required: true
      private_key:
        description:
          - The PEM-encoded private key for the certificate.
          - Must be base64 encoded.
          - Required only for IMC certificate type.
        type: str
      enabled:
        description:
          - Whether the certificate is enabled.
        type: bool
        default: true
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Certificate Management Policy with Root CA certificate
  cisco.intersight.intersight_certificate_management_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "CertMgmt-Policy-01"
    description: "Certificate management policy with Root CA"
    certificates:
      - certificate_type: rootca
        certificate_name: "RootCA-01"
        certificate: "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
        enabled: true
    state: present

- name: Create Certificate Management Policy with IMC certificate
  cisco.intersight.intersight_certificate_management_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "CertMgmt-Policy-02"
    description: "Certificate management policy with IMC certificate"
    certificates:
      - certificate_type: imc
        certificate: "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
        private_key: "LS0tLS1CRUdJTiBSU0EgUFJJVkFURSBLRVktLS0tLQ..."
        enabled: true
    state: present

- name: Create Certificate Management Policy with both types
  cisco.intersight.intersight_certificate_management_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "CertMgmt-Policy-03"
    description: "Certificate management policy with both certificate types"
    certificates:
      - certificate_type: rootca
        certificate_name: "RootCA-01"
        certificate: "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
        enabled: true
      - certificate_type: imc
        certificate: "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
        private_key: "LS0tLS1CRUdJTiBSU0EgUFJJVkFURSBLRVktLS0tLQ..."
        enabled: true
    state: present

- name: Update policy - remove all certificates
  cisco.intersight.intersight_certificate_management_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "CertMgmt-Policy-01"
    certificates: []
    state: present

- name: Delete Certificate Management Policy
  cisco.intersight.intersight_certificate_management_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "CertMgmt-Policy-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "CertMgmt-Policy-01",
        "ObjectType": "certificatemanagement.Policy",
        "Moid": "1234567890abcdef12345678",
        "Description": "Certificate management policy",
        "Certificates": [
            {
                "Certificate": {
                    "PemCertificate": "LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0t..."
                },
                "CertificateName": "RootCA-01",
                "ObjectType": "certificatemanagement.RootCaCertificate",
                "Enabled": true
            }
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec
import re


def validate_certificate_name(certificate_name):
    """
    Validate certificate name according to Intersight constraints.
    """
    if not certificate_name:
        return False, "Certificate name is required for Root CA certificates"

    if len(certificate_name) > 30:
        return False, "Certificate name cannot be more than 30 characters"

    # Check if it starts and ends with alphanumeric
    if not re.match(r'^[a-zA-Z0-9].*[a-zA-Z0-9]$', certificate_name):
        return False, "Certificate name must start and end with an alphanumeric character"

    # Check if it only contains allowed characters (alphanumeric, underscore, hyphen)
    if not re.match(r'^[a-zA-Z0-9_-]+$', certificate_name):
        return False, "Certificate name can only contain alphanumeric characters, underscores, and hyphens"

    return True, ""


def build_certificate_object(cert_config):
    """
    Build certificate object for the Certificates array in policy API body.
    """
    cert_type = cert_config['certificate_type']

    cert_obj = {
        'Enabled': cert_config.get('enabled', True)
    }

    if cert_type == 'rootca':
        cert_obj['ObjectType'] = 'certificatemanagement.RootCaCertificate'
        cert_obj['CertificateName'] = cert_config['certificate_name']
    elif cert_type == 'imc':
        cert_obj['ObjectType'] = 'certificatemanagement.Imc'
        if cert_config.get('private_key'):
            cert_obj['Privatekey'] = cert_config['private_key']
            cert_obj['IsPrivatekeySet'] = True

    # Add certificate data
    if cert_config.get('certificate'):
        cert_obj['Certificate'] = {
            'PemCertificate': cert_config['certificate']
        }

    return cert_obj


def main():
    certificate_options = dict(
        certificate_type=dict(type='str', choices=['rootca', 'imc'], required=True),
        certificate_name=dict(type='str'),
        certificate=dict(type='str', required=True),
        private_key=dict(type='str', no_log=True),
        enabled=dict(type='bool', default=True)
    )

    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        certificates=dict(type='list', elements='dict', options=certificate_options)
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/certificatemanagement/Policies'

    # Validate certificates configuration
    certificates = module.params.get('certificates') or []
    imc_count = 0

    for cert_config in certificates:
        cert_type = cert_config['certificate_type']

        # Count IMC certificates
        if cert_type == 'imc':
            imc_count += 1

        # Validate Root CA certificate name
        if cert_type == 'rootca':
            if not cert_config.get('certificate_name'):
                module.fail_json(msg="certificate_name is required for Root CA certificates")

            is_valid, error_msg = validate_certificate_name(cert_config['certificate_name'])
            if not is_valid:
                module.fail_json(msg=f"Invalid certificate name: {error_msg}")

        # Validate IMC certificate requirements
        if cert_type == 'imc':
            if not cert_config.get('private_key'):
                module.fail_json(msg="private_key is required for IMC certificates")

    # Validate only one IMC certificate per policy
    if imc_count > 1:
        module.fail_json(msg="Only one IMC certificate can be configured per policy")

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Build Certificates array
        if certificates:
            certificates_array = []
            for cert_config in certificates:
                cert_obj = build_certificate_object(cert_config)
                certificates_array.append(cert_obj)
            intersight.api_body['Certificates'] = certificates_array
        else:
            # Empty array to clear all certificates
            intersight.api_body['Certificates'] = []

    # Configure the policy
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()
